@@ -30,6 +30,8 @@
 #include "actioncollection.h"
 #include "mpdconnection.h"
 #include "statuslabel.h"
+#include "messagebox.h"
+#include "stdactions.h"
 
 DynamicPage::DynamicPage(QWidget *p)
     : QWidget(p)
@@ -37,8 +39,9 @@ DynamicPage::DynamicPage(QWidget *p)
     setupUi(this);
     refreshAction = ActionCollection::get()->createAction("refreshdynamic", i18n("Refresh Dynamic Rules"), "view-refresh");
     addAction = ActionCollection::get()->createAction("adddynamic", i18n("Add Dynamic Rules"), "document-new");
-    editAction = ActionCollection::get()->createAction("editdynamic", i18n("Edit Dynamic Rules"), Icons::editIcon);
+    editAction = ActionCollection::get()->createAction("editdynamic", i18n("Edit Dynamic Rules"), Icons::self()->editIcon);
     removeAction = ActionCollection::get()->createAction("removedynamic", i18n("Remove Dynamic Rules"), "list-remove");
+    searchButton->setDefaultAction(StdActions::self()->searchAction);
     toggleAction = new Action(this);
 
     refreshBtn->setDefaultAction(refreshAction);
@@ -64,21 +67,22 @@ DynamicPage::DynamicPage(QWidget *p)
     connect(Dynamic::self()->stopAct(), SIGNAL(triggered()), SLOT(stop()));
     connect(toggleAction, SIGNAL(triggered()), SLOT(toggle()));
     connect(Dynamic::self(), SIGNAL(running(bool)), SLOT(running(bool)));
+    connect(Dynamic::self(), SIGNAL(remoteRunning(bool)), SLOT(remoteRunning(bool)));
     connect(Dynamic::self(), SIGNAL(loadingList()), view, SLOT(showSpinner()));
     connect(Dynamic::self(), SIGNAL(loadedList()), view, SLOT(hideSpinner()));
 
     #ifdef Q_OS_WIN
     infoLabel->setType(StatusLabel::Error);
-    setEnabled(false);
+    enableWidgets(false);
     #else
     infoLabel->setVisible(false);
     refreshBtn->setVisible(false);
     #endif
+    remoteRunningLabel->setType(StatusLabel::Error);
+    remoteRunningLabel->setVisible(false);
     Dynamic::self()->stopAct()->setEnabled(false);
     proxy.setSourceModel(Dynamic::self());
-    view->setTopText(i18n("Dynamic"));
     view->setModel(&proxy);
-    view->hideBackButton();
     view->setDeleteAction(removeAction);
     view->setMode(ItemView::Mode_List);
     controlActions();
@@ -110,10 +114,14 @@ void DynamicPage::dynamicUrlChanged(const QString &url)
 {
     #ifdef Q_OS_WIN
     infoLabel->setVisible(url.isEmpty());
-    setEnabled(!url.isEmpty());
+    enableWidgets(!url.isEmpty());
     #else
     refreshBtn->setVisible(!url.isEmpty());
+    if (url.isEmpty()) {
+        enableWidgets(true);
+    }
     #endif
+    remoteRunningLabel->setVisible(false);
 }
 
 void DynamicPage::add()
@@ -138,9 +146,12 @@ void DynamicPage::remove()
 {
     QModelIndexList selected=view->selectedIndexes();
 
-    if (selected.isEmpty()) {
+    if (selected.isEmpty() ||
+        MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to remove the selected rules?\nThis cannot be undone."),
+                                                 i18n("Remove Dynamic Rules"), StdGuiItem::remove(), StdGuiItem::cancel())) {
         return;
     }
+
     QStringList names;
     foreach (const QModelIndex &idx, selected) {
         names.append(idx.data(Qt::DisplayRole).toString());
@@ -181,3 +192,27 @@ void DynamicPage::running(bool status)
 {
     Dynamic::self()->stopAct()->setEnabled(status);
 }
+
+void DynamicPage::remoteRunning(bool status)
+{
+    remoteRunningLabel->setVisible(!status);
+    enableWidgets(status);
+}
+
+void DynamicPage::enableWidgets(bool enable)
+{
+    controls->setEnabled(enable);
+    view->setEnabled(enable);
+}
+
+void DynamicPage::showEvent(QShowEvent *e)
+{
+    Dynamic::self()->enableRemotePolling(true);
+    QWidget::showEvent(e);
+}
+
+void DynamicPage::hideEvent(QHideEvent *e)
+{
+    Dynamic::self()->enableRemotePolling(false);
+    QWidget::hideEvent(e);
+}
