@@ -49,41 +49,34 @@
 #include <QPainter>
 #include <QFont>
 #include <QXmlStreamReader>
-#include <QDomDocument>
-#include <QDomElement>
 #include <QTimer>
 
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KStandardDirs>
 #include <KDE/KGlobal>
 #include <KDE/KTemporaryFile>
-#include <KDE/KIO/NetAccess>
 #include <QApplication>
 K_GLOBAL_STATIC(Covers, instance)
 #endif
 
+#include <QDebug>
+static bool debugEnabled=false;
+#define DBUG_CLASS(CLASS) if (debugEnabled) qWarning() << CLASS << QThread::currentThread()->objectName() << __FUNCTION__
+#define DBUG DBUG_CLASS(metaObject()->className())
+void Covers::enableDebug()
+{
+    debugEnabled=true;
+}
+
 const QLatin1String Covers::constLastFmApiKey("11172d35eb8cc2fd33250a9e45a2d486");
 const QLatin1String Covers::constCoverDir("covers/");
 const QLatin1String Covers::constCddaCoverDir("cdda/");
 const QLatin1String Covers::constFileName("cover");
 const QLatin1String Covers::constArtistImage("artist");
 
-static const QStringList   constExtensions=QStringList() << ".jpg" << ".png";
-static QStringList coverFileNames;
+static const char * constExtensions[]={".jpg", ".png", 0};
 static bool saveInMpdDir=true;
-
-void initCoverNames()
-{
-    if (coverFileNames.isEmpty()) {
-        QStringList fileNames;
-        fileNames << Covers::constFileName << QLatin1String("AlbumArt") << QLatin1String("folder");
-        foreach (const QString &fileName, fileNames) {
-            foreach (const QString &ext, constExtensions) {
-                coverFileNames << fileName+ext;
-            }
-        }
-    }
-}
+static QString constCoverInTagPrefix=QLatin1String("{tag}");
 
 static bool canSaveTo(const QString &dir)
 {
@@ -94,9 +87,9 @@ static bool canSaveTo(const QString &dir)
 static const QString typeFromRaw(const QByteArray &raw)
 {
     if (raw.size()>9 && /*raw[0]==0xFF && raw[1]==0xD8 && raw[2]==0xFF*/ raw[6]=='J' && raw[7]=='F' && raw[8]=='I' && raw[9]=='F') {
-        return constExtensions.at(0);
+        return constExtensions[0];
     } else if (raw.size()>4 && /*raw[0]==0x89 &&*/ raw[1]=='P' && raw[2]=='N' && raw[3]=='G') {
-        return constExtensions.at(1);
+        return constExtensions[1];
     }
     return QString();
 }
@@ -172,6 +165,14 @@ QString Covers::artistFileName(const Song &song)
     return constArtistImage;
 }
 
+QString Covers::fixArtist(const QString &artist)
+{
+    if (QLatin1String("AC-DC")==artist) {
+        return QLatin1String("AC/DC");
+    }
+    return artist;
+}
+
 #if !defined Q_OS_WIN
 static QString xdgConfig()
 {
@@ -232,7 +233,7 @@ static Covers::Image otherAppCover(const CoverDownloader::Job &job)
                 QString mimeType=typeFromRaw(raw);
                 QString savedName;
                 QString coverName=Covers::albumFileName(job.song);
-                savedName=save(mimeType, mimeType.isEmpty() ? constExtensions.at(0) : mimeType, job.dir+coverName, app.img, raw);
+                savedName=save(mimeType, mimeType.isEmpty() ? constExtensions[0] : mimeType, job.dir+coverName, app.img, raw);
                 if (!savedName.isEmpty()) {
                     app.fileName=savedName;
                 }
@@ -261,8 +262,7 @@ Covers * Covers::self()
 
 bool Covers::isCoverFile(const QString &file)
 {
-    initCoverNames();
-    return coverFileNames.contains(file);
+    return standardNames().contains(file);
 }
 
 static bool fExists(const QString &dir, const QString &file)
@@ -294,22 +294,21 @@ bool Covers::copyImage(const QString &sourceDir, const QString &destDir, const Q
 bool Covers::copyCover(const Song &song, const QString &sourceDir, const QString &destDir, const QString &name, unsigned short maxSize)
 {
     // First, check if dir already has a cover file!
-    initCoverNames();
-    QStringList names=coverFileNames;
-    foreach (const QString &ext, constExtensions) {
-        names+=song.album+ext;
+    QStringList names=standardNames();
+    for (int e=0; constExtensions[e]; ++e) {
+        names+=song.album+constExtensions[e];
     }
 
     QString mpdCover=albumFileName(song);
 
-    foreach (const QString &ext, constExtensions) {
-        if (!names.contains(mpdCover+ext)) {
-            names.prepend(mpdCover+ext);
+    for (int e=0; constExtensions[e]; ++e) {
+        if (!names.contains(mpdCover+constExtensions[e])) {
+            names.prepend(mpdCover+constExtensions[e]);
         }
     }
 
-    foreach (const QString &ext, constExtensions) {
-        names+=song.albumArtist()+QLatin1String(" - ")+song.album+ext;
+    for (int e=0; constExtensions[e]; ++e) {
+        names+=song.albumArtist()+QLatin1String(" - ")+song.album+constExtensions[e];
     }
     foreach (const QString &coverFile, names) {
         if (fExists(destDir, coverFile)) {
@@ -322,8 +321,8 @@ bool Covers::copyCover(const Song &song, const QString &sourceDir, const QString
         if (fExists(sourceDir, coverFile)) {
             QString destName(name);
             if (destName.isEmpty()) { // copying into mpd dir, so we want cover.jpg/png...
-                if (coverFileNames.at(0)!=coverFile) { // source is not 'cover.xxx'
-                    QString ext(coverFile.endsWith(constExtensions.at(0)) ? constExtensions.at(0) : constExtensions.at(1));
+                if (standardNames().at(0)!=coverFile) { // source is not 'cover.xxx'
+                    QString ext(coverFile.endsWith(constExtensions[0]) ? constExtensions[0] : constExtensions[1]);
                     destName=mpdCover+ext;
                 } else {
                     destName=coverFile;
@@ -341,9 +340,9 @@ bool Covers::copyCover(const Song &song, const QString &sourceDir, const QString
         QString artist=encodeName(song.albumArtist());
         QString album=encodeName(song.album);
         QString dir(Utils::cacheDir(constCoverDir+artist, false));
-        foreach (const QString &ext, constExtensions) {
-            if (QFile::exists(dir+album+ext)) {
-                copyImage(dir, destDir, album+ext, destName, maxSize);
+        for (int e=0; constExtensions[e]; ++e) {
+            if (QFile::exists(dir+album+constExtensions[e])) {
+                copyImage(dir, destDir, album+constExtensions[e], destName, maxSize);
                 return true;
             }
         }
@@ -352,31 +351,39 @@ bool Covers::copyCover(const Song &song, const QString &sourceDir, const QString
     return false;
 }
 
-QStringList Covers::standardNames()
+const QStringList & Covers::standardNames()
 {
-    initCoverNames();
-    return coverFileNames;
+    static QStringList *coverFileNames;
+    if (!coverFileNames) {
+        coverFileNames=new QStringList();
+        QStringList fileNames;
+        fileNames << Covers::constFileName << QLatin1String("AlbumArt") << QLatin1String("folder");
+        foreach (const QString &fileName, fileNames) {
+            for (int e=0; constExtensions[e]; ++e) {
+                *coverFileNames << fileName+constExtensions[e];
+            }
+        }
+    }
+
+    return *coverFileNames;
 }
 
 CoverDownloader::CoverDownloader()
 {
     manager=new NetworkAccessManager(this);
-    #ifndef ENABLE_KDE_SUPPORT // KIO is not thread safe!!!
     thread=new Thread(metaObject()->className());
     moveToThread(thread);
     thread->start();
-    #endif
 }
 
 void CoverDownloader::stop()
 {
-    #ifndef ENABLE_KDE_SUPPORT
     thread->stop();
-    #endif
 }
 
 void CoverDownloader::download(const Song &song)
 {
+    DBUG << song.file << song.artist << song.albumartist << song.album;
     bool isArtistImage=song.isArtistImageRequest();
 
     if (jobs.end()!=findJob(Job(song, QString(), isArtistImage))) {
@@ -413,6 +420,7 @@ void CoverDownloader::downloadOnlineImage(Job &job)
     QNetworkReply *j=manager->get(QNetworkRequest(QUrl(job.song.name)));
     connect(j, SIGNAL(finished()), this, SLOT(jobFinished()));
     jobs.insert(j, job);
+    DBUG << job.song.name;
 }
 
 bool CoverDownloader::downloadViaHttp(Job &job, JobType type)
@@ -420,7 +428,7 @@ bool CoverDownloader::downloadViaHttp(Job &job, JobType type)
     QUrl u;
     bool isArtistImage=job.song.isArtistImageRequest();
     QString coverName=isArtistImage ? Covers::artistFileName(job.song) : Covers::albumFileName(job.song);
-    coverName+=constExtensions.at(JobHttpJpg==type ? 0 : 1);
+    coverName+=constExtensions[JobHttpJpg==type ? 0 : 1];
     QString dir=Utils::getDir(job.song.file);
     if (isArtistImage) {
         if (job.level) {
@@ -445,57 +453,35 @@ bool CoverDownloader::downloadViaHttp(Job &job, JobType type)
     QNetworkReply *j=manager->get(QNetworkRequest(u));
     connect(j, SIGNAL(finished()), this, SLOT(jobFinished()));
     jobs.insert(j, job);
+    DBUG << u.toString();
     return true;
 }
 
-static QDomElement addArg(QDomDocument &doc, const QString &key, const QString &val)
-{
-    QDomElement member = doc.createElement("member");
-    QDomElement name = doc.createElement("name");
-    QDomElement value = doc.createElement("value");
-    QDomElement str = doc.createElement("string");
-
-    member.appendChild(name);
-    name.appendChild(doc.createTextNode(key));
-    member.appendChild(value);
-    value.appendChild(str);
-    str.appendChild(doc.createTextNode(val));
-    return member;
-}
-
 void CoverDownloader::downloadViaLastFm(Job &job)
 {
-    // Query lastfm...
-    QDomDocument doc;
-    doc.appendChild(doc.createProcessingInstruction("xml", QString("version=\"1.0\" encoding=\"UTF-8\"")));
-    QDomElement methodCall = doc.createElement("methodCall");
-    QDomElement methodName = doc.createElement("methodName");
-    QDomElement params = doc.createElement("params");
-    QDomElement param = doc.createElement("param");
-    QDomElement value = doc.createElement("value");
-    QDomElement str = doc.createElement("struct");
+    QUrl url("http://ws.audioscrobbler.com/2.0/");
+    #if QT_VERSION < 0x050000
+    QUrl &query=url;
+    #else
+    QUrlQuery query;
+    #endif
 
-    doc.appendChild(methodCall);
-    methodCall.appendChild(methodName);
-    methodName.appendChild(doc.createTextNode(job.isArtist ? "artist.getInfo" : "album.getInfo"));
-    methodCall.appendChild(params);
-    params.appendChild(param);
-    param.appendChild(value);
-    value.appendChild(str);
-
-    str.appendChild(addArg(doc, "api_key", Covers::constLastFmApiKey));
-    str.appendChild(addArg(doc, "artist", job.song.albumArtist()));
+    query.addQueryItem("method", job.isArtist ? "artist.getInfo" : "album.getInfo");
+    query.addQueryItem("api_key", Covers::constLastFmApiKey);
+    query.addQueryItem("autocorrect", "1");
+    query.addQueryItem("artist", Covers::fixArtist(job.song.albumArtist()));
     if (!job.isArtist) {
-        str.appendChild(addArg(doc, "album", job.song.album));
+        query.addQueryItem("album", job.song.album);
     }
+    #if QT_VERSION >= 0x050000
+    url.setQuery(query);
+    #endif
 
-    QNetworkRequest request;
-    request.setUrl(QUrl("http://ws.audioscrobbler.com/2.0/"));
-    request.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml");
-    QNetworkReply *j = manager->post(request, doc.toString().toUtf8());
+    QNetworkReply *j = manager->get(url);
     connect(j, SIGNAL(finished()), this, SLOT(lastFmCallFinished()));
     job.type=JobLastFm;
     jobs.insert(j, job);
+    DBUG << url.toString();
 }
 
 void CoverDownloader::lastFmCallFinished()
@@ -505,6 +491,8 @@ void CoverDownloader::lastFmCallFinished()
         return;
     }
 
+    DBUG << "status" << reply->error() << reply->errorString();
+
     QHash<QNetworkReply *, Job>::Iterator it(jobs.find(reply));
     QHash<QNetworkReply *, Job>::Iterator end(jobs.end());
 
@@ -514,60 +502,35 @@ void CoverDownloader::lastFmCallFinished()
         QString url;
 
         if(QNetworkReply::NoError==reply->error()) {
-            QString resp;
-            QXmlStreamReader doc(QString::fromUtf8(reply->readAll()));
-            int level=0;
+            QXmlStreamReader doc(reply->readAll());
+            QString largeUrl;
+            bool inSection=false;
+            bool isArtistImage=job.isArtist;
 
-            while (!doc.atEnd() && url.isEmpty()) {
+            while (!doc.atEnd()) {
                 doc.readNext();
 
                 if (doc.isStartElement()) {
-                    ++level;
-                    if ( (1==level && QLatin1String("methodResponse")!=doc.name()) ||
-                         (2==level && QLatin1String("params")!=doc.name()) ||
-                         (3==level && QLatin1String("param")!=doc.name()) ||
-                         (4==level && QLatin1String("value")!=doc.name()) ||
-                         (5==level && QLatin1String("string")!=doc.name()) ) {
-                        break;
-                    } else if (5==level) {
-                        resp = doc.readElementText();
-                        break;
+                    if (!inSection && QLatin1String(isArtistImage ? "artist" : "album")==doc.name()) {
+                        inSection=true;
+                    } else if (inSection && QLatin1String("image")==doc.name()) {
+                        QString size=doc.attributes().value("size").toString();
+                        if (QLatin1String("extralarge")==size) {
+                            url = doc.readElementText();
+                        } else if (QLatin1String("large")==size) {
+                            largeUrl = doc.readElementText();
+                        }
+                        if (!url.isEmpty() && !largeUrl.isEmpty()) {
+                            break;
+                        }
                     }
-                } else if (doc.isEndElement()) {
+                } else if (doc.isEndElement() && inSection && QLatin1String(isArtistImage ? "artist" : "album")==doc.name()) {
                     break;
                 }
             }
 
-            if (!resp.isEmpty()) {
-                QXmlStreamReader subDoc(resp.replace("\\\"", "\""));
-                QString largeUrl;
-                bool inSection=false;
-                bool isArtistImage=job.isArtist;
-                while (!subDoc.atEnd()) {
-                    subDoc.readNext();
-
-                    if (subDoc.isStartElement()) {
-                        if (!inSection && QLatin1String(isArtistImage ? "artist" : "album")==subDoc.name()) {
-                            inSection=true;
-                        } else if (inSection && QLatin1String("image")==subDoc.name()) {
-                            QString size=subDoc.attributes().value("size").toString();
-                            if (QLatin1String("extralarge")==size) {
-                                url = subDoc.readElementText();
-                            } else if (QLatin1String("large")==size) {
-                                largeUrl = subDoc.readElementText();
-                            }
-                            if (!url.isEmpty() && !largeUrl.isEmpty()) {
-                                break;
-                            }
-                        }
-                    } else if (subDoc.isEndElement() && inSection && QLatin1String(isArtistImage ? "artist" : "album")==subDoc.name()) {
-                        break;
-                    }
-                }
-
-                if (url.isEmpty() && !largeUrl.isEmpty()) {
-                    url=largeUrl;
-                }
+            if (url.isEmpty() && !largeUrl.isEmpty()) {
+                url=largeUrl;
             }
         }
 
@@ -580,14 +543,18 @@ void CoverDownloader::lastFmCallFinished()
             #endif
             QNetworkReply *j=manager->get(QNetworkRequest(u));
             connect(j, SIGNAL(finished()), this, SLOT(jobFinished()));
+            DBUG << "download" << u.toString();
             jobs.insert(j, job);
         } else {
             if (job.isArtist) {
+                DBUG << "Failed to download artist image";
                 emit artistImage(job.song, QImage(), QString());
             } else {
                 #if defined Q_OS_WIN
+                DBUG << "Failed to download cover image";
                 emit cover(job.song, QImage(), QString());
                 #else
+                DBUG << "Failed to download cover image - try other app";
                 Covers::Image img=otherAppCover(job);
                 emit cover(job.song, img.img, img.fileName);
                 #endif
@@ -604,6 +571,8 @@ void CoverDownloader::jobFinished()
         return;
     }
 
+    DBUG << "status" << reply->error() << reply->errorString();
+
     QHash<QNetworkReply *, Job>::Iterator it(jobs.find(reply));
     QHash<QNetworkReply *, Job>::Iterator end(jobs.end());
 
@@ -637,8 +606,10 @@ void CoverDownloader::jobFinished()
             }
 
             if (job.isArtist) {
+                DBUG << "artist image, null?" << img.img.isNull();
                 emit artistImage(job.song, img.img, img.fileName);
             } else if (img.img.isNull()) {
+                DBUG << "failed to download cover image";
                 #if defined Q_OS_WIN
                 emit cover(job.song, QImage(), QString());
                 #else
@@ -646,6 +617,7 @@ void CoverDownloader::jobFinished()
                 emit cover(job.song, img.img, img.fileName);
                 #endif
             } else {
+                DBUG << "got cover image" << img.fileName;
                 emit cover(job.song, img.img, img.fileName);
             }
         }
@@ -657,7 +629,7 @@ void CoverDownloader::jobFinished()
 QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteArray &raw)
 {
     QString mimeType=typeFromRaw(raw);
-    QString extension=mimeType.isEmpty() ? constExtensions.at(0) : mimeType;
+    QString extension=mimeType.isEmpty() ? constExtensions[0] : mimeType;
     QString savedName;
 
     if (job.song.isCdda()) {
@@ -665,6 +637,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
         if (!dir.isEmpty()) {
             savedName=save(mimeType, extension, dir+job.song.file.mid(7), img, raw);
             if (!savedName.isEmpty()) {
+                DBUG << job.song.file << savedName;
                 return savedName;
             }
         }
@@ -675,6 +648,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
         // ONLINE: Cache dir is saved in Song.title
         savedName=save(mimeType, extension, Utils::cacheDir(job.song.title)+Covers::encodeName(job.isArtist ? job.song.albumartist : (job.song.albumartist+" - "+job.song.album)), img, raw);
         if (!savedName.isEmpty()) {
+            DBUG << job.song.file << savedName;
             return savedName;
         }
     } else if (job.isArtist) {
@@ -685,6 +659,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
                 d.cdUp();
                 savedName=save(mimeType, extension, d.absolutePath()+'/'+Covers::artistFileName(job.song), img, raw);
                 if (!savedName.isEmpty()) {
+                    DBUG << job.song.file << savedName;
                     return savedName;
                 }
             }
@@ -694,6 +669,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
         if (!dir.isEmpty()) {
             savedName=save(mimeType, extension, dir+Covers::encodeName(job.song.albumartist), img, raw);
             if (!savedName.isEmpty()) {
+                DBUG << job.song.file << savedName;
                 return savedName;
             }
         }
@@ -703,6 +679,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
             QString coverName=Covers::albumFileName(job.song);
             savedName=save(mimeType, extension, job.dir+coverName, img, raw);
             if (!savedName.isEmpty()) {
+                DBUG << job.song.file << savedName;
                 return savedName;
             }
         }
@@ -712,6 +689,7 @@ QString CoverDownloader::saveImg(const Job &job, const QImage &img, const QByteA
         if (!dir.isEmpty()) {
             savedName=save(mimeType, extension, dir+Covers::encodeName(job.song.album), img, raw);
             if (!savedName.isEmpty()) {
+                DBUG << job.song.file << savedName;
                 return savedName;
             }
         }
@@ -779,10 +757,12 @@ void CoverLocator::locate()
     }
     QList<LocatedCover> covers;
     foreach (const Song &s, toDo) {
+        DBUG << s.file << s.artist << s.albumartist << s.album;
         Covers::Image img=Covers::locateImage(s);
         covers.append(LocatedCover(s, img.img, img.fileName));
     }
     if (!covers.isEmpty()) {
+        DBUG << "located" << covers.count();
         emit located(covers);
     }
 
@@ -912,12 +892,22 @@ Covers::Image Covers::findImage(const Song &song, bool emitResult)
 
 Covers::Image Covers::locateImage(const Song &song)
 {
+    DBUG_CLASS("Covers") << song.file << song.artist << song.albumartist << song.album;
     bool isArtistImage=song.isArtistImageRequest();
     QString prevFileName=Covers::self()->getFilename(song, isArtistImage);
     if (!prevFileName.isEmpty()) {
+        #ifdef TAGLIB_FOUND
+        QImage img;
+        if (prevFileName.startsWith(constCoverInTagPrefix)) {
+            img=Tags::readImage(prevFileName.mid(constCoverInTagPrefix.length()));
+        } else {
+            img=QImage(prevFileName);
+        }
+        #else
         QImage img(prevFileName);
-
+        #endif
         if (!img.isNull()) {
+            DBUG_CLASS("Covers") << "Found previous" << prevFileName;
             return Image(img, prevFileName);
         }
     }
@@ -927,15 +917,17 @@ Covers::Image Covers::locateImage(const Song &song)
     if (isOnline) {
         // ONLINE: Cache dir is saved in Song.title
         QString baseName=Utils::cacheDir(song.title, false)+encodeName(isArtistImage ? song.albumartist : (song.albumartist+" - "+song.album));
-        foreach (const QString &ext, constExtensions) {
-            if (QFile::exists(baseName+ext)) {
-                QImage img(baseName+ext);
+        for (int e=0; constExtensions[e]; ++e) {
+            if (QFile::exists(baseName+constExtensions[e])) {
+                QImage img(baseName+constExtensions[e]);
 
                 if (!img.isNull()) {
-                    return Image(img, baseName+ext);
+                    DBUG_CLASS("Covers") << "Got online from cache" << QString(baseName+constExtensions[e]);
+                    return Image(img, baseName+constExtensions[e]);
                 }
             }
         }
+        DBUG_CLASS("Covers") << "Online image not in cache";
         return Image(QImage(), QString());
     }
 
@@ -965,6 +957,7 @@ Covers::Image Covers::locateImage(const Song &song)
                         QImage img(dirName+fileName);
 
                         if (!img.isNull()) {
+                            DBUG_CLASS("Covers") << "Got artist image" << QString(dirName+fileName);
                             return Image(img, dirName+fileName);
                         }
                     }
@@ -974,29 +967,29 @@ Covers::Image Covers::locateImage(const Song &song)
                 dirName=Utils::fixPath(d.absolutePath());
             }
         } else {
-            initCoverNames();
             QStringList names;
             QString mpdCover=albumFileName(song);
             if (!mpdCover.isEmpty()) {
-                foreach (const QString &ext, constExtensions) {
-                    names << mpdCover+ext;
+                for (int e=0; constExtensions[e]; ++e) {
+                    names << mpdCover+constExtensions[e];
                 }
             }
-            names << coverFileNames;
-            foreach (const QString &ext, constExtensions) {
-                names+=Utils::changeExtension(Utils::getFile(song.file), ext);
+            names << standardNames();
+            for (int e=0; constExtensions[e]; ++e) {
+                names+=Utils::changeExtension(Utils::getFile(song.file), constExtensions[e]);
             }
-            foreach (const QString &ext, constExtensions) {
-                names+=song.albumArtist()+QLatin1String(" - ")+song.album+ext;
+            for (int e=0; constExtensions[e]; ++e) {
+                names+=song.albumArtist()+QLatin1String(" - ")+song.album+constExtensions[e];
             }
-            foreach (const QString &ext, constExtensions) {
-                names+=song.album+ext;
+            for (int e=0; constExtensions[e]; ++e) {
+                names+=song.album+constExtensions[e];
             }
             foreach (const QString &fileName, names) {
                 if (QFile::exists(dirName+fileName)) {
                     QImage img(dirName+fileName);
 
                     if (!img.isNull()) {
+                        DBUG_CLASS("Covers") << "Got cover image" << QString(dirName+fileName);
                         return Image(img, dirName+fileName);
                     }
                 }
@@ -1007,7 +1000,8 @@ Covers::Image Covers::locateImage(const Song &song)
             if (QFile::exists(fileName)) {
                 QImage img(Tags::readImage(fileName));
                 if (!img.isNull()) {
-                    return Image(img, QString());
+                    DBUG_CLASS("Covers") << "Got cover image from tag" << fileName;
+                    return Image(img, constCoverInTagPrefix+fileName);
                 }
             }
             #endif
@@ -1017,6 +1011,7 @@ Covers::Image Covers::locateImage(const Song &song)
                 QImage img(dirName+fileName);
 
                 if (!img.isNull()) {
+                    DBUG_CLASS("Covers") << "Got cover image" << QString(dirName+fileName);
                     return Image(img, dirName+fileName);
                 }
             }
@@ -1027,11 +1022,12 @@ Covers::Image Covers::locateImage(const Song &song)
 
     if (isArtistImage) {
         QString dir(Utils::cacheDir(constCoverDir, false));
-        foreach (const QString &ext, constExtensions) {
-            if (QFile::exists(dir+artist+ext)) {
-                QImage img(dir+artist+ext);
+        for (int e=0; constExtensions[e]; ++e) {
+            if (QFile::exists(dir+artist+constExtensions[e])) {
+                QImage img(dir+artist+constExtensions[e]);
                 if (!img.isNull()) {
-                    return Image(img, dir+artist+ext);
+                    DBUG_CLASS("Covers") << "Got artist image" << QString(dir+artist+constExtensions[e]);
+                    return Image(img, dir+artist+constExtensions[e]);
                 }
             }
         }
@@ -1039,11 +1035,12 @@ Covers::Image Covers::locateImage(const Song &song)
         QString album=encodeName(song.album);
         // Check if cover is already cached
         QString dir(Utils::cacheDir(constCoverDir+artist, false));
-        foreach (const QString &ext, constExtensions) {
-            if (QFile::exists(dir+album+ext)) {
-                QImage img(dir+album+ext);
+        for (int e=0; constExtensions[e]; ++e) {
+            if (QFile::exists(dir+album+constExtensions[e])) {
+                QImage img(dir+album+constExtensions[e]);
                 if (!img.isNull()) {
-                    return Image(img, dir+album+ext);
+                    DBUG_CLASS("Covers") << "Got cover image" << QString(dir+album+constExtensions[e]);
+                    return Image(img, dir+album+constExtensions[e]);
                 }
             }
         }
@@ -1054,16 +1051,25 @@ Covers::Image Covers::locateImage(const Song &song)
         // See if amarok, or clementine, has it...
         Image app=otherAppCover(job);
         if (!app.img.isNull()) {
+            DBUG_CLASS("Covers") << "Got cover image (other app)" << app.fileName;
             return app;
         }
         #endif
     }
 
+    DBUG_CLASS("Covers") << "Failed to locate image";
     return Image(QImage(), QString());
 }
 
+// Dont return song files as cover files!
+static Covers::Image fix(const Covers::Image &img)
+{
+    return Covers::Image(img.img, img.fileName.startsWith(constCoverInTagPrefix) ? QString() : img.fileName);
+}
+
 Covers::Image Covers::requestImage(const Song &song)
 {
+    DBUG << song.file << song.artist << song.albumartist << song.album;
     if (retrieved>=constMaxPerLoopIteration) {
         emit locate(song);
         return Covers::Image();
@@ -1072,9 +1078,10 @@ Covers::Image Covers::requestImage(const Song &song)
     retrieved++;
     Image img=findImage(song, false);
     if (img.img.isNull()) {
+        DBUG << song.file << song.artist << song.albumartist << song.album << "Need to download";
         emit download(song);
     }
-    return img;
+    return fix(img);
 }
 
 void Covers::located(const QList<LocatedCover> &covers)
@@ -1135,7 +1142,8 @@ void Covers::gotAlbumCover(const Song &song, const QImage &img, const QString &f
         mutex.unlock();
     }
     if (emitResult) {
-        emit cover(song, img, fileName);
+        DBUG << "emit cover" << song.file << song.artist << song.albumartist << song.album << img.width() << img.height() << fileName;
+        emit cover(song, img, fileName.startsWith(constCoverInTagPrefix) ? QString() : fileName);
     }
 }
 
@@ -1147,7 +1155,8 @@ void Covers::gotArtistImage(const Song &song, const QImage &img, const QString &
         mutex.unlock();
     }
     if (emitResult) {
-        emit artistImage(song, img, fileName);
+        DBUG << "emit artistImage" << song.file << song.artist << song.albumartist << song.album << img.width() << img.height() << fileName;
+        emit artistImage(song, img, fileName.startsWith(constCoverInTagPrefix) ? QString() : fileName);
     }
 }
 
