@@ -98,9 +98,7 @@ private:
     QHash<QNetworkReply *, Job> jobs;
 
 private:
-    #ifndef ENABLE_KDE_SUPPORT
     Thread *thread;
-    #endif
     NetworkAccessManager *manager;
 };
 
@@ -154,6 +152,8 @@ public:
         QString fileName;
     };
 
+    static void enableDebug();
+
     static const QSize constMaxSize;
     static const QLatin1String constLastFmApiKey;
     static const QLatin1String constCoverDir;
@@ -165,10 +165,11 @@ public:
     static bool isCoverFile(const QString &file);
     static bool copyImage(const QString &sourceDir, const QString &destDir, const QString &coverFile, const QString &destName, unsigned short maxSize=0);
     static bool copyCover(const Song &song, const QString &sourceDir, const QString &destDir, const QString &name=QString(), unsigned short maxSize=0);
-    static QStringList standardNames();
+    static const QStringList &standardNames();
     static QString encodeName(QString name);
     static QString albumFileName(const Song &song);
     static QString artistFileName(const Song &song);
+    static QString fixArtist(const QString &artist);
 
     Covers();
     void stop();
