@@ -45,6 +45,7 @@ LibraryPage::LibraryPage(QWidget *p)
     addToPlayQueue->setDefaultAction(StdActions::self()->addToPlayQueueAction);
     replacePlayQueue->setDefaultAction(StdActions::self()->replacePlayQueueAction);
     libraryUpdate->setDefaultAction(StdActions::self()->refreshAction);
+    searchButton->setDefaultAction(StdActions::self()->searchAction);
 
     view->addAction(StdActions::self()->addToPlayQueueAction);
     view->addAction(StdActions::self()->replacePlayQueueAction);
@@ -80,7 +81,6 @@ LibraryPage::LibraryPage(QWidget *p)
     connect(view, SIGNAL(searchItems()), this, SLOT(searchItems()));
     connect(view, SIGNAL(rootIndexSet(QModelIndex)), this, SLOT(updateGenres(QModelIndex)));
     proxy.setSourceModel(MusicLibraryModel::self());
-    view->setTopText(i18n("Artists"));
     view->setModel(&proxy);
 }
 
@@ -208,7 +208,8 @@ void LibraryPage::deleteSongs()
     QList<Song> songs=selectedSongs();
 
     if (!songs.isEmpty()) {
-        if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to remove the selected songs?\nThis cannot be undone."))) {
+        if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to delete the selected songs?\nThis cannot be undone."),
+                                                      i18n("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
             emit deleteSongs(QString(), songs);
         }
         view->clearSelection();
@@ -255,7 +256,7 @@ void LibraryPage::showSongs(const QList<Song> &songs)
 
 void LibraryPage::showArtist(const QString &artist)
 {
-    QModelIndex idx=MusicLibraryModel::self()->findArtistsIndex(artist);
+    QModelIndex idx=MusicLibraryModel::self()->findArtistIndex(artist);
     if (idx.isValid()) {
         idx=proxy.mapFromSource(idx);
         genreCombo->setCurrentIndex(0);
@@ -267,6 +268,22 @@ void LibraryPage::showArtist(const QString &artist)
     }
 }
 
+void LibraryPage::showAlbum(const QString &artist, const QString &album)
+{
+    QModelIndex idx=MusicLibraryModel::self()->findAlbumIndex(artist, album);
+    if (idx.isValid()) {
+        idx=proxy.mapFromSource(idx);
+        genreCombo->setCurrentIndex(0);
+        view->clearSearchText();
+        view->showIndex(idx, true);
+        if (ItemView::Mode_SimpleTree==view->viewMode() || ItemView::Mode_DetailedTree==view->viewMode()) {
+            view->setExpanded(idx.parent());
+            view->setExpanded(idx);
+        }
+    }
+}
+
+
 void LibraryPage::itemDoubleClicked(const QModelIndex &)
 {
     const QModelIndexList selected = view->selectedIndexes();
