@@ -43,14 +43,17 @@ PlaylistsPage::PlaylistsPage(QWidget *p)
     renamePlaylistAction = ActionCollection::get()->createAction("renameplaylist", i18n("Rename"), "edit-rename");
     replacePlayQueue->setDefaultAction(StdActions::self()->replacePlayQueueAction);
     libraryUpdate->setDefaultAction(StdActions::self()->refreshAction);
+    searchButton->setDefaultAction(StdActions::self()->searchAction);
     connect(genreCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(searchItems()));
     connect(PlaylistsModel::self(), SIGNAL(updateGenres(const QSet<QString> &)), genreCombo, SLOT(update(const QSet<QString> &)));
 
     view->allowGroupedView();
-    view->setTopText(i18n("Playlists"));
     view->addAction(StdActions::self()->addToPlayQueueAction);
     view->addAction(StdActions::self()->replacePlayQueueAction);
     view->addAction(StdActions::self()->addWithPriorityAction);
+    #ifdef ENABLE_DEVICES_SUPPORT
+    view->addAction(StdActions::self()->copyToDeviceAction);
+    #endif
     view->addAction(renamePlaylistAction);
     view->addAction(StdActions::self()->removeAction);
     view->setUniformRowHeights(true);
@@ -180,7 +183,8 @@ void PlaylistsPage::removeItems()
     }
 
     if (remPlaylists.count() &&
-        MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to remove the selected playlists?\nThis cannot be undone."), i18n("Remove Playlists?"))) {
+        MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to remove the selected playlists?\nThis cannot be undone."),
+                                                 i18n("Remove Playlists"), StdGuiItem::remove(), StdGuiItem::cancel())) {
             return;
     }
 
@@ -200,7 +204,8 @@ void PlaylistsPage::savePlaylist()
 
     if (!name.isEmpty()) {
         if (PlaylistsModel::self()->exists(name)) {
-            if (MessageBox::No==MessageBox::warningYesNo(this, i18n("A playlist named <b>%1</b> already exists!<br/>Overwrite?").arg(name), i18n("Overwrite Playlist?"))) {
+            if (MessageBox::No==MessageBox::warningYesNo(this, i18n("A playlist named <b>%1</b> already exists!<br/>Overwrite?", name),
+                                                         i18n("Overwrite Playlist"), StdGuiItem::overwrite(), StdGuiItem::cancel())) {
                 return;
             }
             else {
@@ -222,7 +227,8 @@ void PlaylistsPage::renamePlaylist()
 
         if (!newName.isEmpty() && name!=newName) {
             if (PlaylistsModel::self()->exists(newName)) {
-                if (MessageBox::No==MessageBox::warningYesNo(this, i18n("A playlist named <b>%1</b> already exists!<br/>Overwrite?").arg(newName), i18n("Overwrite Playlist?"))) {
+                if (MessageBox::No==MessageBox::warningYesNo(this, i18n("A playlist named <b>%1</b> already exists!<br/>Overwrite?", newName),
+                                                             i18n("Overwrite Playlist"), StdGuiItem::overwrite(), StdGuiItem::cancel())) {
                     return;
                 }
                 else {
@@ -282,24 +288,55 @@ void PlaylistsPage::addItemsToPlayQueue(const QModelIndexList &indexes, bool rep
     }
 }
 
+#ifdef ENABLE_DEVICES_SUPPORT
+QList<Song> PlaylistsPage::selectedSongs() const
+{
+    QModelIndexList selected = view->selectedIndexes();
+
+    if (0==selected.size()) {
+        return QList<Song>();
+    }
+
+    QModelIndexList mapped;
+    foreach (const QModelIndex &idx, selected) {
+        mapped.append(proxy.mapToSource(idx));
+    }
+
+    return PlaylistsModel::self()->songs(mapped);
+}
+
+void PlaylistsPage::addSelectionToDevice(const QString &udi)
+{
+    QList<Song> songs=selectedSongs();
+    if (!songs.isEmpty()) {
+        emit addToDevice(QString(), udi, songs);
+        view->clearSelection();
+    }
+}
+#endif
+
 void PlaylistsPage::controlActions()
 {
     QModelIndexList selected=view->selectedIndexes();
-    bool enable=false;
+    bool canRename=false;
+    bool enableActions=selected.count()>0;
 
     if (1==selected.count()) {
         QModelIndex index = proxy.mapToSource(selected.first());
         PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(index.internalPointer());
         if (item && item->isPlaylist()) {
-            enable=true;
+            canRename=true;
         }
     }
-    renamePlaylistAction->setEnabled(enable);
-    renamePlaylistAction->setEnabled(enable);
-    StdActions::self()->removeAction->setEnabled(selected.count()>0);
-    StdActions::self()->replacePlayQueueAction->setEnabled(selected.count()>0);
-    StdActions::self()->addToPlayQueueAction->setEnabled(selected.count()>0);
-    StdActions::self()->addWithPriorityAction->setEnabled(selected.count()>0);
+
+    renamePlaylistAction->setEnabled(canRename);
+    StdActions::self()->removeAction->setEnabled(enableActions);
+    StdActions::self()->replacePlayQueueAction->setEnabled(enableActions);
+    StdActions::self()->addToPlayQueueAction->setEnabled(enableActions);
+    StdActions::self()->addWithPriorityAction->setEnabled(enableActions);
+    #ifdef ENABLE_DEVICES_SUPPORT
+    StdActions::self()->copyToDeviceAction->setEnabled(enableActions);
+    #endif
     menuButton->controlState();
 }
 
