@@ -69,38 +69,55 @@ struct MpdDefaults
         return parts.count()>1 ? parts[1] : QString();
     }
 
+    enum Details {
+        DT_DIR    = 0x01,
+        DT_ADDR   = 0x02,
+        DT_PORT   = 0x04,
+        DT_PASSWD = 0x08,
+
+        DT_ALL    = DT_DIR|DT_ADDR|DT_PORT|DT_PASSWD
+    };
+
     void read() {
         QFile f("/etc/mpd.conf");
+        int details=0;
 
         if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
             while (!f.atEnd()) {
                 QString line = f.readLine().trimmed();
                 if (line.startsWith('#')) {
                     continue;
-                } else if (line.startsWith(QLatin1String("music_directory"))) {
+                } else if (!(details&DT_DIR) && line.startsWith(QLatin1String("music_directory"))) {
                     QString val=getVal(line);
                     if (!val.isEmpty() && QDir(val).exists()) {
                         dir=val;
+                        details|=DT_DIR;
                     }
-                } else if (line.startsWith(QLatin1String("bind_to_address"))) {
+                } else if (!(details&DT_ADDR) && line.startsWith(QLatin1String("bind_to_address"))) {
                     QString val=getVal(line);
                     if (!val.isEmpty() && val!=QLatin1String("any")) {
                         host=val;
+                        details|=DT_ADDR;
                     }
-                } else if (line.startsWith(QLatin1String("port"))) {
+                } else if (!(details&DT_PORT) && line.startsWith(QLatin1String("port"))) {
                     int val=getVal(line).toInt();
                     if (val>0) {
                         port=val;
+                        details|=DT_PORT;
                     }
-                } else if (line.startsWith(QLatin1String("password"))) {
+                } else if (!(details&DT_PASSWD) && line.startsWith(QLatin1String("password"))) {
                     QString val=getVal(line);
                     if (!val.isEmpty()) {
                         QStringList parts=val.split('@');
-                        if (parts.count()) {
+                        if (!parts.isEmpty()) {
                             passwd=parts[0];
+                            details|=DT_PASSWD;
                         }
                     }
                 }
+                if (details==DT_ALL) {
+                    break;
+                }
             }
         }
     }
@@ -257,15 +274,6 @@ QString Settings::iconTheme()
 }
 #endif
 
-bool Settings::monoSidebarIcons()
-{
-    #ifdef Q_OS_WIN
-    return GET_BOOL("monoSidebarIcons", false);
-    #else
-    return GET_BOOL("monoSidebarIcons", true);
-    #endif
-}
-
 bool Settings::showPlaylist()
 {
     return GET_BOOL("showPlaylist", true);
@@ -278,6 +286,9 @@ bool Settings::showFullScreen()
 
 QByteArray Settings::playQueueHeaderState()
 {
+    if (version()<CANTATA_MAKE_VERSION(1, 0, 53)) {
+        return QByteArray();
+    }
     return GET_BYTE_ARRAY("playQueueHeaderState");
 }
 
@@ -326,11 +337,6 @@ bool Settings::stopDynamizerOnExit()
     return GET_BOOL("stopDynamizerOnExit", false);
 }
 
-bool Settings::stopAfterCurrent()
-{
-    return GET_BOOL("stopAfterCurrent", false);
-}
-
 bool Settings::storeCoversInMpdDir()
 {
     return GET_BOOL("storeCoversInMpdDir", true);
@@ -354,9 +360,7 @@ bool Settings::storeStreamsInMpdDir()
 int Settings::libraryView()
 {
     int v=version();
-    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
-                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree)
-                                    : ItemView::Mode_List);
+    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
     return ItemView::toMode(GET_STRING("libraryView", def));
 }
 
@@ -367,24 +371,26 @@ int Settings::albumsView()
 
 int Settings::folderView()
 {
-    return ItemView::toMode(GET_STRING("folderView", ItemView::modeStr(ItemView::Mode_SimpleTree)));
+    int v=version();
+    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(1, 0, 51) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
+    return ItemView::toMode(GET_STRING("folderView", def));
 }
 
 int Settings::playlistsView()
 {
     int v=version();
-    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
-                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_GroupedTree)
-                                    : ItemView::Mode_List);
+    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50)
+                                    ? ItemView::Mode_SimpleTree
+                                    : v<CANTATA_MAKE_VERSION(1, 0, 51)
+                                        ? ItemView::Mode_GroupedTree
+                                        : ItemView::Mode_DetailedTree);
     return ItemView::toMode(GET_STRING("playlistsView", def));
 }
 
 int Settings::streamsView()
 {
     int v=version();
-    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
-                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree)
-                                    : ItemView::Mode_List);
+    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
     return ItemView::toMode(GET_STRING("streamsView", def));
 }
 
@@ -412,9 +418,6 @@ int Settings::albumsCoverSize()
 
 int Settings::albumSort()
 {
-    if (version()<CANTATA_MAKE_VERSION(0, 6, 0)) {
-        return GET_BOOL("albumFirst", true) ? 0 : 1;
-    }
     return GET_INT("albumSort", 0);
 }
 
@@ -461,25 +464,40 @@ QStringList Settings::lyricProviders()
     return GET_STRINGLIST("lyricProviders", def);
 }
 
-int Settings::lyricsZoom()
+QStringList Settings::wikipediaLangs()
 {
-    return GET_INT("lyricsZoom", 0);
+    QStringList def=QStringList() << "en:en";
+    return GET_STRINGLIST("wikipediaLangs", def);
 }
 
-bool Settings::lyricsBgnd()
+bool Settings::wikipediaIntroOnly()
 {
-    return GET_BOOL("lyricsBgnd", true);
+    return GET_BOOL("wikipediaIntroOnly", true);
 }
 
-int Settings::infoZoom()
+bool Settings::contextBackdrop()
 {
-    int zoom=GET_INT("infoZoom", 0);
-    if (zoom>0 && version()<CANTATA_MAKE_VERSION(0, 9, 51)) {
-        modified=true;
-        zoom=0;
-        SET_VALUE("infoZoom", zoom);
-    }
-    return zoom;
+    return GET_BOOL("contextBackdrop", true);
+}
+
+bool Settings::contextDarkBackground()
+{
+    return GET_BOOL("contextDarkBackground", false);
+}
+
+int Settings::contextZoom()
+{
+    return GET_INT("contextZoom", 0);
+}
+
+QString Settings::contextSlimPage()
+{
+    return GET_STRING("contextSlimPage", QString());
+}
+
+QByteArray Settings::contextSplitterState()
+{
+    return GET_BYTE_ARRAY("contextSplitterState");
 }
 
 QString Settings::page()
@@ -489,14 +507,13 @@ QString Settings::page()
 
 QStringList Settings::hiddenPages()
 {
-    QStringList def;
-    def << "FolderPage";
+    QStringList def=QStringList() << "PlayQueuePage" << "FolderPage" << "ContextPage";
     return GET_STRINGLIST("hiddenPages", def);
 }
 
 bool Settings::gnomeMediaKeys()
 {
-    return GET_BOOL("gnomeMediaKeys", false);
+    return GET_BOOL("gnomeMediaKeys", true);
 }
 
 #ifdef ENABLE_DEVICES_SUPPORT
@@ -512,7 +529,14 @@ bool Settings::showDeleteAction()
 
 int Settings::devicesView()
 {
-    return GET_INT("devicesView", (int)ItemView::Mode_DetailedTree);
+    if (version()<CANTATA_MAKE_VERSION(1, 0, 51)) {
+        int v=GET_INT("devicesView", (int)ItemView::Mode_DetailedTree);
+        modified=true;
+        SET_VALUE("devicesView", ItemView::modeStr((ItemView::Mode)v));
+        return v;
+    } else {
+        return ItemView::toMode(GET_STRING("devicesView", ItemView::modeStr(ItemView::Mode_DetailedTree)));
+    }
 }
 #endif
 
@@ -524,7 +548,7 @@ int Settings::version()
         if (3==parts.size()) {
             ver=CANTATA_MAKE_VERSION(parts.at(0).toInt(), parts.at(1).toInt(), parts.at(2).toInt());
         } else {
-            ver=CANTATA_MAKE_VERSION(0, 9, 0);
+            ver=PACKAGE_VERSION;
             SET_VALUE("version", PACKAGE_VERSION);
         }
     }
@@ -533,9 +557,7 @@ int Settings::version()
 
 int Settings::stopFadeDuration()
 {
-    int def=version()<CANTATA_MAKE_VERSION(0, 4, 0) ? MinFade : DefaultFade;
-    int v=GET_INT("stopFadeDuration", def);
-
+    int v=GET_INT("stopFadeDuration", (int)DefaultFade);
     if (0!=v && (v<MinFade || v>MaxFade)) {
         v=DefaultFade;
     }
@@ -543,31 +565,14 @@ int Settings::stopFadeDuration()
 }
 
 #ifdef TAGLIB_FOUND
-int Settings::httpPort()
-{
-    return GET_INT("httpPort", 0);
-}
-
 int Settings::httpAllocatedPort()
 {
     return GET_INT("httpAllocatedPort", 0);
 }
 
-QString Settings::httpAddress()
+QString Settings::httpInterface()
 {
-    return GET_STRING("httpAddress", QString());
-}
-
-bool Settings::enableHttp()
-{
-    bool def=version()>CANTATA_MAKE_VERSION(0, 9, 50);
-    return GET_BOOL("enableHttp", def);
-}
-
-bool Settings::alwaysUseHttp()
-{
-    bool def=version()>CANTATA_MAKE_VERSION(0, 9, 50);
-    return GET_BOOL("alwaysUseHttp", def);
+    return GET_STRING("httpInterface", QString());
 }
 #endif
 
@@ -583,7 +588,7 @@ bool Settings::playQueueAutoExpand()
 
 bool Settings::playQueueStartClosed()
 {
-    return GET_BOOL("playQueueStartClosed", version()<CANTATA_MAKE_VERSION(0, 6, 1));
+    return GET_BOOL("playQueueStartClosed", false);
 }
 
 bool Settings::playQueueScroll()
@@ -591,12 +596,17 @@ bool Settings::playQueueScroll()
     return GET_BOOL("playQueueScroll", true);
 }
 
+bool Settings::playQueueBackground()
+{
+    return GET_BOOL("playQueueBackground", false);
+}
+
 bool Settings::playListsStartClosed()
 {
     return GET_BOOL("playListsStartClosed", true);
 }
 
-#ifdef PHONON_FOUND
+#ifdef ENABLE_HTTP_STREAM_PLAYBACK
 bool Settings::playStream()
 {
     return GET_BOOL("playStream", false);
@@ -613,12 +623,16 @@ bool Settings::cdAuto()
 {
     return GET_BOOL("cdAuto", true);
 }
-#ifdef LAME_FOUND
-bool Settings::cdEncode()
+
+bool Settings::paranoiaFull()
 {
-    return GET_BOOL("cdEncode", true);
+    return GET_BOOL("paranoiaFull", true);
+}
+
+bool Settings::paranoiaNeverSkip()
+{
+    return GET_BOOL("paranoiaNeverSkip", true);
 }
-#endif
 #endif
 
 #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
@@ -638,16 +652,6 @@ int Settings::cddbPort()
 {
     return GET_INT("cddbPort", 8880);
 }
-
-bool Settings::paranoiaFull()
-{
-    return GET_BOOL("paranoiaFull", true);
-}
-
-bool Settings::paranoiaNeverSkip()
-{
-    return GET_BOOL("paranoiaNeverSkip", true);
-}
 #endif
 
 bool Settings::forceSingleClick()
@@ -660,14 +664,18 @@ bool Settings::startHidden()
     return GET_BOOL("startHidden", false);
 }
 
-QString Settings::amazonAccessKey()
+bool Settings::monoSidebarIcons()
 {
-    return GET_STRING("amazonAccessKey", QString());
+    #ifdef Q_OS_WIN
+    return GET_BOOL("monoSidebarIcons", false);
+    #else
+    return GET_BOOL("monoSidebarIcons", true);
+    #endif
 }
 
-QString Settings::amazonSecretAccessKey()
+bool Settings::showTimeRemaining()
 {
-    return GET_STRING("amazonSecretAccessKey", QString());
+    return GET_BOOL("showTimeRemaining", false);
 }
 
 void Settings::removeConnectionDetails(const QString &v)
@@ -794,11 +802,6 @@ void Settings::saveStopDynamizerOnExit(bool v)
     SET_VALUE_MOD(stopDynamizerOnExit)
 }
 
-void Settings::saveStopAfterCurrent(bool v)
-{
-    SET_VALUE_MOD(stopAfterCurrent)
-}
-
 void Settings::saveStoreCoversInMpdDir(bool v)
 {
     SET_VALUE_MOD(storeCoversInMpdDir)
@@ -889,19 +892,39 @@ void Settings::saveLyricProviders(const QStringList &v)
     SET_VALUE_MOD(lyricProviders)
 }
 
-void Settings::saveLyricsZoom(int v)
+void Settings::saveWikipediaLangs(const QStringList &v)
 {
-    SET_VALUE_MOD(lyricsZoom)
+    SET_VALUE_MOD(wikipediaLangs)
 }
 
-void Settings::saveLyricsBgnd(bool v)
+void Settings::saveWikipediaIntroOnly(bool v)
 {
-    SET_VALUE_MOD(lyricsBgnd)
+    SET_VALUE_MOD(wikipediaIntroOnly)
 }
 
-void Settings::saveInfoZoom(int v)
+void Settings::saveContextBackdrop(bool v)
 {
-    SET_VALUE_MOD(infoZoom)
+    SET_VALUE_MOD(contextBackdrop)
+}
+
+void Settings::saveContextDarkBackground(bool v)
+{
+    SET_VALUE_MOD(contextDarkBackground)
+}
+
+void Settings::saveContextZoom(int v)
+{
+    SET_VALUE_MOD(contextZoom)
+}
+
+void Settings::saveContextSlimPage(const QString &v)
+{
+    SET_VALUE_MOD(contextSlimPage)
+}
+
+void Settings::saveContextSplitterState(const QByteArray &v)
+{
+    SET_VALUE_MOD(contextSplitterState)
 }
 
 void Settings::savePage(const QString &v)
@@ -932,7 +955,7 @@ void Settings::saveShowDeleteAction(bool v)
 
 void Settings::saveDevicesView(int v)
 {
-    SET_VALUE_MOD(devicesView)
+    SET_ITEMVIEW_MODE_VALUE_MOD(devicesView)
 }
 #endif
 
@@ -947,29 +970,14 @@ void Settings::saveStopFadeDuration(int v)
 }
 
 #ifdef TAGLIB_FOUND
-void Settings::saveHttpPort(int v)
-{
-    SET_VALUE_MOD(httpPort)
-}
-
 void Settings::saveHttpAllocatedPort(int v)
 {
     SET_VALUE_MOD(httpAllocatedPort)
 }
 
-void Settings::saveHttpAddress(const QString &v)
+void Settings::saveHttpInterface(const QString &v)
 {
-    SET_VALUE_MOD(httpAddress)
-}
-
-void Settings::saveEnableHttp(bool v)
-{
-    SET_VALUE_MOD(enableHttp)
-}
-
-void Settings::saveAlwaysUseHttp(bool v)
-{
-    SET_VALUE_MOD(alwaysUseHttp)
+    SET_VALUE_MOD(httpInterface)
 }
 #endif
 
@@ -993,12 +1001,17 @@ void Settings::savePlayQueueScroll(bool v)
     SET_VALUE_MOD(playQueueScroll)
 }
 
+void Settings::savePlayQueueBackground(bool v)
+{
+    SET_VALUE_MOD(playQueueBackground)
+}
+
 void Settings::savePlayListsStartClosed(bool v)
 {
     SET_VALUE_MOD(playListsStartClosed)
 }
 
-#ifdef PHONON_FOUND
+#ifdef ENABLE_HTTP_STREAM_PLAYBACK
 void Settings::savePlayStream(bool v)
 {
     SET_VALUE_MOD(playStream)
@@ -1015,12 +1028,16 @@ void Settings::saveCdAuto(bool v)
 {
     SET_VALUE_MOD(cdAuto)
 }
-#ifdef LAME_FOUND
-void Settings::saveCdEncode(bool v)
+
+void Settings::saveParanoiaFull(bool v)
 {
-    SET_VALUE_MOD(cdEncode)
+    SET_VALUE_MOD(paranoiaFull)
+}
+
+void Settings::saveParanoiaNeverSkip(bool v)
+{
+    SET_VALUE_MOD(paranoiaNeverSkip)
 }
-#endif
 #endif
 
 #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
@@ -1040,16 +1057,6 @@ void Settings::saveCddbPort(int v)
 {
     SET_VALUE_MOD(cddbPort)
 }
-
-void Settings::saveParanoiaFull(bool v)
-{
-    SET_VALUE_MOD(paranoiaFull)
-}
-
-void Settings::saveParanoiaNeverSkip(bool v)
-{
-    SET_VALUE_MOD(paranoiaNeverSkip)
-}
 #endif
 
 void Settings::saveForceSingleClick(bool v)
@@ -1062,6 +1069,16 @@ void Settings::saveStartHidden(bool v)
     SET_VALUE_MOD(startHidden);
 }
 
+void Settings::saveMonoSidebarIcons(bool v)
+{
+    SET_VALUE_MOD(monoSidebarIcons);
+}
+
+void Settings::saveShowTimeRemaining(bool v)
+{
+    SET_VALUE_MOD(showTimeRemaining);
+}
+
 void Settings::save(bool force)
 {
     if (force) {
