@@ -26,7 +26,9 @@
 #include <QMimeData>
 #include <QStringList>
 #include <QPainter>
+#include <QFile>
 #include "localize.h"
+#include "qtplural.h"
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobal>
 #endif
@@ -43,6 +45,14 @@
 #include "itemview.h"
 #include "mpdparseutils.h"
 #include "icons.h"
+#include "utils.h"
+
+#ifdef CACHE_SCALED_COVERS
+static QString cacheCoverName(const QString &artist, const QString &album, int size, bool createDir=false)
+{
+    return Utils::cacheDir(Covers::constCoverDir+QString::number(size)+"/"+Covers::encodeName(artist), createDir)+Covers::encodeName(album)+".png";
+}
+#endif
 
 static int sortAlbums=AlbumsModel::Sort_AlbumArtist;
 
@@ -159,10 +169,9 @@ QModelIndex AlbumsModel::parent(const QModelIndex &index) const
 
     Item *item=static_cast<Item *>(index.internalPointer());
 
-    if(item->isAlbum())
+    if (item->isAlbum()) {
         return QModelIndex();
-    else
-    {
+    } else {
         SongItem *song=static_cast<SongItem *>(item);
 
         if (song->parent) {
@@ -213,7 +222,7 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
             int iSize=iconSize();
 
             if (Qt::DecorationRole==role && 0==iSize) {
-                return Icons::albumIcon;
+                return Icons::self()->albumIcon;
             }
 
             if (!theDefaultIcon) {
@@ -222,7 +231,7 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
                 if (0==cSize) {
                     cSize=stdSize=22;
                 }
-                theDefaultIcon = new QPixmap(Icons::albumIcon.pixmap(stdSize, stdSize)
+                theDefaultIcon = new QPixmap(Icons::self()->albumIcon.pixmap(stdSize, stdSize)
                                             .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             }
             if (!al->coverRequested && iSize && Song::SingleTracks!=al->type) {
@@ -240,13 +249,11 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
                 ? al->name
                 : (year>0 ? QString("%1\n%2 (%3)\n").arg(al->artist).arg(al->album).arg(QString::number(year)) : QString("%1\n%2\n").arg(al->artist).arg(al->album))+
                     #ifdef ENABLE_KDE_SUPPORT
-                    i18np("1 Track (%2)", "%1 Tracks (%2)", al->songs.count(), Song::formattedTime(al->totalTime()));
+                    i18np("1 Track (%2)", "%1 Tracks (%2)", al->trackCount(), Song::formattedTime(al->totalTime(), true));
                     #else
-                    QTP_TRACKS_DURATION_STR(al->songs.count(), Song::formattedTime(al->totalTime()));
+                    QTP_TRACKS_DURATION_STR(al->trackCount(), Song::formattedTime(al->totalTime(), true));
                     #endif
         }
-        case ItemView::Role_Search:
-            return al->album;
         case Qt::DisplayRole:
             return al->name;
         case ItemView::Role_MainText:
@@ -256,6 +263,8 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         }
         case ItemView::Role_SubText:
             return Sort_AlbumArtist==sortAlbums ? al->artist : al->album;
+        case ItemView::Role_TitleText:
+            return i18nc("Album by Artist", "%1 by %2", al->album, al->artist);
         case Qt::SizeHintRole:
             if (!itemSize.isNull()) {
                 return itemSize;
@@ -268,18 +277,18 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         default:
             return ActionModel::data(index, role);
         case Qt::DecorationRole:
-            return Icon(Song::Playlist==si->type ? "view-media-playlist" : "audio-x-generic");
+            return Song::Playlist==si->type ? Icons::self()->playlistIcon : Icons::self()->audioFileIcon;
         case Qt::ToolTipRole: {
             quint32 year=si->parent->songs.count() ? si->parent->songs.at(0)->year : 0;
             return si->parent->artist+QLatin1String("<br/>")+
                    si->parent->album+(year>0 ? (QLatin1String(" (")+QString::number(year)+QChar(')')) : QString())+QLatin1String("<br/>")+
                    data(index, Qt::DisplayRole).toString()+QLatin1String("<br/>")+
-                   (Song::Playlist==si->type ? QString() : Song::formattedTime(si->time)+QLatin1String("<br/>"))+
+                   Song::formattedTime(si->time, true)+QLatin1String("<br/>")+
                    QLatin1String("<small><i>")+si->file+QLatin1String("</i></small>");
         }
         case Qt::DisplayRole:
             if (Song::Playlist==si->type) {
-                return si->title;
+                return si->file.endsWith(".cue", Qt::CaseInsensitive) ? i18n("Cue Sheet") : i18n("Playlist");
             }
             if (Song::SingleTracks==si->parent->type) {
                 return si->artistSong();
@@ -287,9 +296,8 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
             else {
                 return si->trackAndTitleStr(Song::isVariousArtists(si->parent->artist) && !Song::isVariousArtists(si->artist));
             }
-        case ItemView::Role_SubText: {
-            return Song::formattedTime(si->time);
-        }
+        case ItemView::Role_SubText:
+            return Song::formattedTime(si->time, true);
         }
     }
 
@@ -459,11 +467,12 @@ void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &f
 
     QList<AlbumItem *>::Iterator it=items.begin();
     QList<AlbumItem *>::Iterator end=items.end();
+    QString artist=Song::MultipleArtists==song.type ? i18n("Various Artists") : song.albumArtist();
 
     for (int row=0; it!=end; ++it, ++row) {
-        if ((*it)->artist==song.albumArtist() && (*it)->album==song.album) {
+        if ((*it)->artist==artist && (*it)->album==song.album) {
             if (!(*it)->cover || update) {
-                (*it)->cover=new QPixmap(QPixmap::fromImage(img.scaled(QSize(iconSize(), iconSize()), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+                (*it)->setCover(img);
                 QModelIndex idx=index(row, 0, QModelIndex());
                 emit dataChanged(idx, idx);
             }
@@ -534,6 +543,7 @@ AlbumsModel::AlbumItem::AlbumItem(const QString &ar, const QString &al, quint16
     , cover(0)
     , updated(false)
     , coverRequested(false)
+    , numTracks(0)
     , time(0)
 {
     setName();
@@ -580,14 +590,29 @@ void AlbumsModel::AlbumItem::setName()
             : (artist+QLatin1String(" - ")+album);
 }
 
+quint32 AlbumsModel::AlbumItem::trackCount()
+{
+    updateStats();
+    return numTracks;
+}
+
 quint32 AlbumsModel::AlbumItem::totalTime()
+{
+    updateStats();
+    return time;
+}
+
+void AlbumsModel::AlbumItem::updateStats()
 {
     if (0==time) {
+        numTracks=0;
         foreach (SongItem *s, songs) {
-            time+=s->time;
+            if (Song::Playlist!=s->type) {
+                time+=s->time;
+                numTracks++;
+            }
         }
     }
-    return time;
 }
 
 void AlbumsModel::AlbumItem::getCover()
@@ -603,24 +628,54 @@ void AlbumsModel::AlbumItem::getCover()
         s.album=album;
         s.year=year;
         s.file=firstSong->file;
+        s.type=type;
+        #ifdef CACHE_SCALED_COVERS
+        int size=iconSize();
+        QString cache=cacheCoverName(s.artist, s.album, size);
+
+        if (QFile::exists(cache)) {
+            QImage img(cache);
+            if (!img.isNull()) {
+                cover=new QPixmap(QPixmap::fromImage(img));
+                return;
+            }
+        }
+        #endif
         Covers::Image img=Covers::self()->requestImage(s);
         if (!img.img.isNull()) {
-            cover=new QPixmap(QPixmap::fromImage(img.img.scaled(QSize(iconSize(), iconSize()), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+            setCover(img.img);
         }
     }
 }
 
+void AlbumsModel::AlbumItem::setCover(const QImage &img)
+{
+    #ifdef CACHE_SCALED_COVERS
+    if (Song::SingleTracks!=type && songs.count() && !img.isNull()) {
+        SongItem *firstSong=songs.first();
+        Song s;
+        if (Song::MultipleArtists==type) {  // Then Cantata has placed this album under 'Various Artists' but the actual album as a different AlbumArtist tag
+            s.artist=firstSong->albumArtist();
+        } else {
+            s.artist=artist;
+        }
+        s.album=album;
+        int size=iconSize();
+        QImage scaled=img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+        scaled.save(cacheCoverName(s.artist, s.album, size, true));
+        cover=new QPixmap(QPixmap::fromImage(scaled));
+    }
+    #else
+    int size=iconSize();
+    cover=new QPixmap(QPixmap::fromImage(img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+    #endif
+}
+
 const AlbumsModel::SongItem *AlbumsModel::AlbumItem::getCueFile() const
 {
-    if (2==songs.count()) {
-        const SongItem *a=songs.at(0);
-        const SongItem *b=songs.at(1);
-
-        if ( ( (Song::Playlist==a->type && Song::Playlist!=b->type) ||
-               (Song::Playlist!=a->type && Song::Playlist==b->type) ) &&
-             ( (Song::Playlist==a->type && a->file.endsWith(".cue", Qt::CaseInsensitive)) ||
-               (Song::Playlist==b->type && b->file.endsWith(".cue", Qt::CaseInsensitive)) ) ) {
-            return Song::Playlist==a->type ? a : b;
+    foreach (SongItem *s, songs) {
+        if (Song::Playlist==s->type && s->file.endsWith(".cue", Qt::CaseInsensitive)) {
+            return s;
         }
     }
 
