@@ -36,14 +36,12 @@
 #include "device.h"
 #include "utils.h"
 #endif
-#ifdef ENABLE_ONLINE_SERVICES
 #include "onlineservice.h"
-#endif
 #include <QPixmap>
 #include <QApplication>
 #include <QFontMetrics>
+#include <QFile>
 
-#include <QDebug>
 static MusicLibraryItemAlbum::CoverSize coverSize=MusicLibraryItemAlbum::CoverNone;
 static QPixmap *theDefaultIcon=0;
 static QPixmap *theDefaultLargeIcon=0;
@@ -62,6 +60,13 @@ static inline int adjust(int v)
 
 static int fontHeight=16;
 
+#ifdef CACHE_SCALED_COVERS
+static QString cacheCoverName(const QString &artist, const QString &album, int size, bool createDir=false)
+{
+    return Utils::cacheDir(Covers::constCoverDir+QString::number(size)+"/"+Covers::encodeName(artist), createDir)+Covers::encodeName(album)+".png";
+}
+#endif
+
 void MusicLibraryItemAlbum::setup()
 {
     fontHeight=QApplication::fontMetrics().height();
@@ -158,6 +163,7 @@ MusicLibraryItemAlbum::MusicLibraryItemAlbum(const QString &data, quint32 year,
     , m_year(year)
     , m_yearOfTrack(0xFFFF)
     , m_yearOfDisc(0xFFFF)
+    , m_totalTime(0)
     , m_coverIsDefault(false)
     , m_cover(0)
     , m_type(Song::Standard)
@@ -172,8 +178,12 @@ MusicLibraryItemAlbum::~MusicLibraryItemAlbum()
 void MusicLibraryItemAlbum::setCoverImage(const QImage &img) const
 {
     int size=iconSize(largeImages());
-    m_cover = new QPixmap(QPixmap::fromImage(img).scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
+    QImage scaled=img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    m_cover = new QPixmap(QPixmap::fromImage(scaled));
     m_coverIsDefault=false;
+    #ifdef CACHE_SCALED_COVERS
+    scaled.save(cacheCoverName(parentItem()->data(), data(), size, true));
+    #endif
 }
 
 bool MusicLibraryItemAlbum::setCover(const QImage &img, bool update) const
@@ -208,15 +218,28 @@ const QPixmap & MusicLibraryItemAlbum::cover()
                 cSize=22;
             }
             if (useLarge) {
-                theDefaultLargeIcon = new QPixmap(Icons::albumIcon.pixmap(cSize, cSize)
+                theDefaultLargeIcon = new QPixmap(Icons::self()->albumIcon.pixmap(cSize, cSize)
                                                  .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             } else {
-                theDefaultIcon = new QPixmap(Icons::albumIcon.pixmap(cSize, cSize)
+                theDefaultIcon = new QPixmap(Icons::self()->albumIcon.pixmap(cSize, cSize)
                                             .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             }
         }
         m_coverIsDefault = true;
         if (Song::SingleTracks!=m_type && parentItem() && iSize && childCount()) {
+            #ifdef CACHE_SCALED_COVERS
+            QString cache=cacheCoverName(parentItem()->data(), data(), iSize);
+
+            if (QFile::exists(cache)) {
+                QImage img(cache);
+                if (!img.isNull()) {
+                    m_cover=new QPixmap(QPixmap::fromImage(img));
+                    m_coverIsDefault=false;
+                    return *m_cover;
+                }
+            }
+            #endif
+
             MusicLibraryItemSong *firstSong=static_cast<MusicLibraryItemSong*>(childItem(0));
             Song song;
             if (Song::MultipleArtists==m_type) { // Then Cantata has placed this album under 'Various Artists' but the actual album as a different AlbumArtist tag
@@ -228,6 +251,7 @@ const QPixmap & MusicLibraryItemAlbum::cover()
             song.album=m_itemData;
             song.year=m_year;
             song.file=firstSong->file();
+            song.type=m_type;
             Covers::Image img;
             #ifdef ENABLE_DEVICES_SUPPORT
             if (parentItem() && parentItem()->parentItem() && dynamic_cast<Device *>(parentItem()->parentItem()) &&
@@ -237,16 +261,15 @@ const QPixmap & MusicLibraryItemAlbum::cover()
                 static_cast<Device *>(parentItem()->parentItem())->requestCover(song);
             } else
             #endif
-            #ifdef ENABLE_ONLINE_SERVICES
             if (parentItem() && parentItem()->parentItem() && dynamic_cast<OnlineService *>(parentItem()->parentItem()) &&
                 static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
                 // ONLINE: Image URL is encoded in song.name...
-                song.name=m_imageUrl;
-                song.title=parentItem()->parentItem()->data().toLower();
-                img=Covers::self()->requestImage(song);
-            } else
-            #endif
-            if (parentItem() && parentItem()->parentItem() && !static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
+                if (!m_imageUrl.isEmpty()) {
+                    song.name=m_imageUrl;
+                    song.title=parentItem()->parentItem()->data().toLower();
+                    img=Covers::self()->requestImage(song);
+                }
+            } else if (parentItem() && parentItem()->parentItem() && !static_cast<MusicLibraryItemRoot *>(parentItem()->parentItem())->useAlbumImages()) {
                 // Not showing album images in this model, so dont request any!
             } else {
                 img=Covers::self()->requestImage(song);
@@ -264,13 +287,29 @@ const QPixmap & MusicLibraryItemAlbum::cover()
 }
 
 quint32 MusicLibraryItemAlbum::totalTime()
+{
+    updateStats();
+    return m_totalTime;
+}
+
+quint32 MusicLibraryItemAlbum::trackCount()
+{
+    updateStats();
+    return m_numTracks;
+}
+
+void MusicLibraryItemAlbum::updateStats()
 {
     if (0==m_totalTime) {
+        m_numTracks=0;
         foreach (MusicLibraryItem *i, m_childItems) {
-            m_totalTime+=static_cast<MusicLibraryItemSong *>(i)->time();
+            MusicLibraryItemSong *song=static_cast<MusicLibraryItemSong *>(i);
+            if (Song::Playlist!=song->song().type) {
+                m_totalTime+=song->time();
+                m_numTracks++;
+            }
         }
     }
-    return m_totalTime;
 }
 
 void MusicLibraryItemAlbum::addTracks(MusicLibraryItemAlbum *other)
@@ -312,6 +351,7 @@ void MusicLibraryItemAlbum::append(MusicLibraryItem *i)
         song->song().type=Song::MultipleArtists;
     }
     m_totalTime=0;
+    m_artists.clear();
 }
 
 void MusicLibraryItemAlbum::remove(int row)
@@ -326,6 +366,8 @@ void MusicLibraryItemAlbum::remove(int row)
     }
     delete i;
     m_totalTime=0;
+    m_artists.clear();
+    resetRows();
 }
 
 void MusicLibraryItemAlbum::remove(MusicLibraryItemSong *i)
@@ -334,6 +376,40 @@ void MusicLibraryItemAlbum::remove(MusicLibraryItemSong *i)
     if (-1!=idx) {
         remove(idx);
     }
+    resetRows();
+}
+
+void MusicLibraryItemAlbum::removeAll(const QSet<QString> &fileNames)
+{
+    QSet<QString> fn=fileNames;
+    for (int i=0; i<m_childItems.count() && !fn.isEmpty();) {
+        MusicLibraryItemSong *song=static_cast<MusicLibraryItemSong *>(m_childItems.at(i));
+        if (fn.contains(song->file())) {
+            fn.remove(song->file());
+            delete m_childItems.takeAt(i);
+            m_totalTime=0;
+            m_artists.clear();
+        } else {
+            ++i;
+        }
+    }
+    resetRows();
+}
+
+QMap<QString, Song> MusicLibraryItemAlbum::getSongs(const QSet<QString> &fileNames) const
+{
+    QMap<QString, Song> map;
+    foreach (const MusicLibraryItem *i, m_childItems) {
+        const MusicLibraryItemSong *song=static_cast<const MusicLibraryItemSong *>(i);
+        if (fileNames.contains(song->file())) {
+            map.insert(song->file(), song->song());
+            if (map.size()==fileNames.size()) {
+                return map;
+            }
+        }
+    }
+
+    return map;
 }
 
 bool MusicLibraryItemAlbum::detectIfIsMultipleArtists()
@@ -358,15 +434,10 @@ bool MusicLibraryItemAlbum::detectIfIsMultipleArtists()
 
 const MusicLibraryItemSong * MusicLibraryItemAlbum::getCueFile() const
 {
-    if (Song::Standard==m_type && 2==m_childItems.count()) {
-        const MusicLibraryItemSong *a=static_cast<const MusicLibraryItemSong *>(m_childItems.at(0));
-        const MusicLibraryItemSong *b=static_cast<const MusicLibraryItemSong *>(m_childItems.at(1));
-
-        if ( ( (Song::Playlist==a->song().type && Song::Playlist!=b->song().type) ||
-               (Song::Playlist!=a->song().type && Song::Playlist==b->song().type) ) &&
-             ( (Song::Playlist==a->song().type && a->song().file.endsWith(".cue", Qt::CaseInsensitive)) ||
-               (Song::Playlist==b->song().type && b->song().file.endsWith(".cue", Qt::CaseInsensitive)) ) ) {
-            return Song::Playlist==a->song().type ? a : b;
+    foreach (const MusicLibraryItem *s, m_childItems) {
+        if (Song::Playlist==static_cast<const MusicLibraryItemSong *>(s)->song().type &&
+            static_cast<const MusicLibraryItemSong *>(s)->song().file.endsWith(".cue", Qt::CaseInsensitive)) {
+            return static_cast<const MusicLibraryItemSong *>(s);
         }
     }
 
@@ -385,6 +456,29 @@ bool MusicLibraryItemAlbum::updateYear()
     return true;
 }
 
+bool MusicLibraryItemAlbum::containsArtist(const QString &a)
+{
+    if (m_artists.isEmpty()) {
+        foreach (MusicLibraryItem *itm, m_childItems) {
+            m_artists.insert(static_cast<MusicLibraryItemSong *>(itm)->song().basicArtist());
+        }
+    }
+
+    return m_artists.contains(a);
+}
+
+void MusicLibraryItemAlbum::clearImage()
+{
+    if (!m_coverIsDefault) {
+        m_coverIsDefault=true;
+        delete m_cover;
+        m_cover=0;
+        if (theDefaultIcon) {
+            m_cover=theDefaultIcon;
+        }
+    }
+}
+
 void MusicLibraryItemAlbum::setYear(const MusicLibraryItemSong *song)
 {
     if (m_childItems.isEmpty() || (m_yearOfDisc>song->disc() || (m_yearOfDisc==song->disc() && m_yearOfTrack>song->track()))) {
