@@ -30,6 +30,7 @@
 #include "playlistsmodel.h"
 #include "playlistsproxymodel.h"
 #include "itemview.h"
+#include "qtplural.h"
 #include "groupedview.h"
 #include "localize.h"
 #include "utils.h"
@@ -200,7 +201,7 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                     QTP_TRACKS_DURATION_STR(pl->songs.count(), Song::formattedTime(pl->totalTime()));
                     #endif
         case Qt::DecorationRole:
-            return Icons::playlistIcon;
+            return Icons::self()->playlistIcon;
         case ItemView::Role_SubText:
             #ifdef ENABLE_KDE_SUPPORT
             return i18np("1 Track (%2)", "%1 Tracks (%2)", pl->songs.count(), Song::formattedTime(pl->totalTime()));
@@ -282,7 +283,7 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             return text;
         }
         case Qt::DecorationRole:
-            return s->title.isEmpty() ? Icons::streamIcon : Icon("audio-x-generic");
+            return s->title.isEmpty() ? Icons::self()->streamIcon : Icon("audio-x-generic");
         case ItemView::Role_MainText:
             return s->title.isEmpty() ? s->file : s->title;
         case ItemView::Role_SubText:
@@ -349,6 +350,34 @@ QStringList PlaylistsModel::filenames(const QModelIndexList &indexes, bool files
     return fnames;
 }
 
+QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
+{
+    QList<Song> songs;
+    QSet<Item *> selectedPlaylists;
+
+    foreach (const QModelIndex &idx, indexes) {
+        Item *item=static_cast<Item *>(idx.internalPointer());
+        if (item->isPlaylist()) {
+            PlaylistItem *playlist=static_cast<PlaylistItem *>(item);
+            foreach (const SongItem *song, playlist->songs) {
+                selectedPlaylists.insert(item);
+                songs.append(*song);
+            }
+        }
+    }
+    foreach (const QModelIndex &idx, indexes) {
+        PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(idx.internalPointer());
+        if (!item->isPlaylist()) {
+            SongItem *song=static_cast<SongItem *>(item);
+            if (!selectedPlaylists.contains(song->parent)) {
+                songs.append(*song);
+            }
+        }
+    }
+    qSort(songs);
+    return songs;
+}
+
 static const QLatin1String constPlaylistNameMimeType("cantata/playlistnames");
 static const QLatin1String constPositionsMimeType("cantata/positions");
 static const char *constPlaylistProp="hasPlaylist";
