@@ -28,6 +28,7 @@
 #include <QString>
 #include <QStringList>
 #include <QUrl>
+#include <QFile>
 #include "localize.h"
 #include "dirviewitemroot.h"
 #include "dirviewitemdir.h"
@@ -47,6 +48,17 @@
 #include "httpserver.h"
 #endif
 #include "utils.h"
+#include "cuefile.h"
+#include "mpdconnection.h"
+#include "onlineservice.h"
+
+#include <QDebug>
+static bool debugEnabled=false;
+#define DBUG if (debugEnabled) qWarning() << "MPDParseUtils"
+void MPDParseUtils::enableDebug()
+{
+    debugEnabled=true;
+}
 
 QList<Playlist> MPDParseUtils::parsePlaylists(const QByteArray &data)
 {
@@ -237,12 +249,14 @@ Song MPDParseUtils::parseSong(const QByteArray &data, bool isPlayQueue)
     if (!song.file.isEmpty()) {
         if (song.isStream()) {
             if (!song.isCantataStream()) {
-                QString name=getAndRemoveStreamName(song.file);
-                if (!name.isEmpty()) {
-                    song.name=name;
-                }
-                if (song.title.isEmpty() && song.name.isEmpty()) {
-                    song.title=Utils::getFile(QUrl(song.file).path());
+                if (!OnlineService::decode(song)) {
+                    QString name=getAndRemoveStreamName(song.file);
+                    if (!name.isEmpty()) {
+                        song.name=name;
+                    }
+                    if (song.title.isEmpty() && song.name.isEmpty()) {
+                        song.title=Utils::getFile(QUrl(song.file).path());
+                    }
                 }
             }
         } else {
@@ -363,8 +377,9 @@ void MPDParseUtils::setGroupMultiple(bool g)
     groupMultipleArtists=g;
 }
 
-MusicLibraryItemRoot * MPDParseUtils::parseLibraryItems(const QByteArray &data)
+MusicLibraryItemRoot * MPDParseUtils::parseLibraryItems(const QByteArray &data, const QString &mpdDir, long mpdVersion)
 {
+    bool canSplitCue=mpdVersion>=MPD_MAKE_VERSION(0,17,0);
     MusicLibraryItemRoot * const rootItem = new MusicLibraryItemRoot;
     QByteArray currentItem;
     QList<QByteArray> lines = data.split('\n');
@@ -372,6 +387,8 @@ MusicLibraryItemRoot * MPDParseUtils::parseLibraryItems(const QByteArray &data)
     MusicLibraryItemArtist *artistItem = 0;
     MusicLibraryItemAlbum *albumItem = 0;
     MusicLibraryItemSong *songItem = 0;
+    QString unknown=i18n("Unknown");
+    QString lastGenre;
 
     for (int i = 0; i < amountOfLines; i++) {
         currentItem += lines.at(i);
@@ -384,9 +401,147 @@ MusicLibraryItemRoot * MPDParseUtils::parseLibraryItems(const QByteArray &data)
             }
 
             if (Song::Playlist==currentSong.type) {
-                if (songItem && Utils::getDir(songItem->file())==Utils::getDir(currentSong.file)) {
+                MusicLibraryItemAlbum *prevAlbum=albumItem;
+                QString prevSongFile=songItem ? songItem->file() : QString();
+                QList<Song> cueSongs; // List of songs from cue file
+                QSet<QString> cueFiles; // List of source (flac, mp3, etc) files referenced in cue file
+
+                DBUG << "Got playlist item" << currentSong.file << "prevFile:" << prevSongFile;
+                if (canSplitCue && currentSong.file.endsWith(".cue", Qt::CaseInsensitive) && !mpdDir.startsWith("http://") &&
+                        CueFile::parse(currentSong.file, mpdDir, cueSongs, cueFiles) &&
+                        (cueFiles.count()<cueSongs.count() || (albumItem && albumItem->data()==unknown && albumItem->parentItem()->data()==unknown))) {
+                    DBUG << "Parsed file, songs:" << cueSongs.count() << "files:" << cueFiles;
+
+                    bool canUseThisCueFile=true;
+                    foreach (const Song &s, cueSongs) {
+                        if (!QFile::exists(mpdDir+s.name)) {
+                            DBUG << QString(mpdDir+s.name) << "is referenced in cue file, but does not exist in MPD folder";
+                            canUseThisCueFile=false;
+                            break;
+                        }
+                    }
+
+                    if (!canUseThisCueFile) {
+                        continue;
+                    }
+
+                    bool canUseCueFileTracks=false;
+                    QList<Song> fixedCueSongs; // Songs taken from cueSongs that have been updated...
+
+                    if (albumItem) {
+                        QMap<QString, Song> origFiles=albumItem->getSongs(cueFiles);
+                        DBUG << "Original files:" << origFiles.keys();
+                        if (origFiles.size()==cueFiles.size()) {
+                            // We have a previous album, if any of the details of the songs from the cue are empty,
+                            // use those from the album...
+                            bool setTimeFromSource=origFiles.size()==cueSongs.size();
+                            quint32 albumTime=1==cueFiles.size() ? albumItem->totalTime() : 0;
+                            quint32 usedAlbumTime=0;
+                            foreach (const Song &orig, cueSongs) {
+                                Song s=orig;
+                                Song albumSong=origFiles[s.name];
+                                s.name=QString(); // CueFile has placed source file name here!
+                                if (s.artist.isEmpty()) {
+                                    s.artist=albumSong.artist;
+                                    DBUG << "Get artist from album" << albumSong.artist;
+                                }
+                                if (s.album.isEmpty()) {
+                                    s.album=albumSong.album;
+                                    DBUG << "Get album from album" << albumSong.album;
+                                }
+                                if (s.albumartist.isEmpty()) {
+                                    s.albumartist=albumSong.albumartist;
+                                    DBUG << "Get albumartist from album" << albumSong.album;
+                                }
+                                if (0==s.year) {
+                                    s.year=albumSong.year;
+                                    DBUG << "Get year from album" << albumSong.year;
+                                }
+                                if (0==s.time && setTimeFromSource) {
+                                    s.time=albumSong.time;
+                                } else if (0!=albumTime) {
+                                    // Try to set duration of last track by subtracting previous track durations from album duration...
+                                    if (0==s.time) {
+                                        s.time=albumTime-usedAlbumTime;
+                                    } else {
+                                        usedAlbumTime+=s.time;
+                                    }
+                                }
+                                fixedCueSongs.append(s);
+                            }
+                            canUseCueFileTracks=true;
+                        } else DBUG << "ERROR: file count mismatch" << origFiles.size() << cueFiles.size();
+                    } else DBUG << "ERROR: No album???";
+
+                    if (!canUseCueFileTracks) {
+                        // No revious album, or album had a different number of source files to the CUE file. If so, then we need to ensure
+                        // all tracks have meta data - otherwise just fallback to listing file + cue
+                        foreach (const Song &orig, cueSongs) {
+                            Song s=orig;
+                            s.name=QString(); // CueFile has placed source file name here!
+                            if (s.artist.isEmpty() || s.album.isEmpty()) {
+                                break;
+                            }
+                            fixedCueSongs.append(s);
+                        }
+
+                        if (fixedCueSongs.count()==cueSongs.count()) {
+                            canUseCueFileTracks=true;
+                        } else DBUG << "ERROR: Not all cue tracks had meta data";
+                    }
+
+                    if (canUseCueFileTracks) {
+                        QSet<MusicLibraryItemAlbum *> updatedAlbums;
+                        foreach (Song s, fixedCueSongs) {
+                            s.fillEmptyFields();
+                            if (!artistItem || s.albumArtist()!=artistItem->data()) {
+                                artistItem = rootItem->artist(s);
+                            }
+                            if (!albumItem || s.year!=albumItem->year() || albumItem->parentItem()!=artistItem || s.album!=albumItem->data()) {
+                                albumItem = artistItem->album(s);
+                            }
+                            DBUG << "Create new track from cue" << s.file << s.title << s.artist << s.albumartist << s.album;
+                            songItem = new MusicLibraryItemSong(s, albumItem);
+                            albumItem->append(songItem);
+                            updatedAlbums.insert(albumItem);
+
+                            if (s.genre!=lastGenre) {
+                                lastGenre=s.genre;
+                                albumItem->addGenre(s.genre);
+                                artistItem->addGenre(s.genre);
+                                rootItem->addGenre(s.genre);
+                            }
+                        }
+
+                        // For each album that was updated/created, remove any source files referenced in cue file...
+                        foreach (MusicLibraryItemAlbum *al, updatedAlbums) {
+                            al->removeAll(cueFiles);
+                            if (prevAlbum && al!=prevAlbum) {
+                                DBUG << "Removing" << cueFiles.count() << " files from " << prevAlbum->data();
+                                prevAlbum->removeAll(cueFiles);
+                            }
+                        }
+
+                        // Remove any artist/album that was created and is now empty.
+                        // This will happen if the source file (e.g. the flac file) does not have any metadata...
+                        if (prevAlbum && 0==prevAlbum->childCount()) {
+                            DBUG << "Removing empty previous album" << prevAlbum->data();
+                            MusicLibraryItemArtist *ar=static_cast<MusicLibraryItemArtist *>(prevAlbum->parentItem());
+                            ar->remove(prevAlbum);
+                            if (0==ar->childCount()) {
+                                rootItem->remove(ar);
+                            }
+                        }
+                    }
+                }
+
+                // Add playlist file (or cue file) to current album, if it has the same path!
+                // This assumes that MPD always send playlists as the last file...
+                if (!prevSongFile.isEmpty() && Utils::getDir(prevSongFile)==Utils::getDir(currentSong.file)) {
                     currentSong.albumartist=currentSong.artist=artistItem->data();
                     currentSong.album=albumItem->data();
+                    currentSong.time=albumItem->totalTime();
+                    DBUG << "Adding playlist file to" << albumItem->parentItem()->data() << albumItem->data();
                     songItem = new MusicLibraryItemSong(currentSong, albumItem);
                     albumItem->append(songItem);
                 }
@@ -403,12 +558,14 @@ MusicLibraryItemRoot * MPDParseUtils::parseLibraryItems(const QByteArray &data)
             if (!albumItem || currentSong.year!=albumItem->year() || albumItem->parentItem()!=artistItem || currentSong.album!=albumItem->data()) {
                 albumItem = artistItem->album(currentSong);
             }
-
             songItem = new MusicLibraryItemSong(currentSong, albumItem);
             albumItem->append(songItem);
-            albumItem->addGenre(currentSong.genre);
-            artistItem->addGenre(currentSong.genre);
-            rootItem->addGenre(currentSong.genre);
+            if (currentSong.genre!=lastGenre) {
+                lastGenre=currentSong.genre;
+                albumItem->addGenre(currentSong.genre);
+                artistItem->addGenre(currentSong.genre);
+                rootItem->addGenre(currentSong.genre);
+            }
         }
     }
 
