@@ -85,7 +85,7 @@ struct Song
     void fillEmptyFields();
     void setKey();
     virtual void clear();
-    static QString formattedTime(quint32 seconds);
+    static QString formattedTime(quint32 seconds, bool zeroIsUnknown=false);
     QString format();
     QString entryName() const;
     QString artistSong() const;
@@ -115,6 +115,8 @@ struct Song
     }
 
     bool isArtistImageRequest() const { return album.isEmpty() && artist.isEmpty() && !albumartist.isEmpty() && 0==size && 0==track; }
+
+    QString basicArtist() const;
 };
 
 Q_DECLARE_METATYPE(Song)
