@@ -33,6 +33,10 @@
 #include "localize.h"
 #include "messagebox.h"
 #include "jobcontroller.h"
+#include "basicitemdelegate.h"
+#include "action.h"
+#include "cuefile.h"
+#include <QComboBox>
 #include <QTreeWidget>
 #include <QLabel>
 #include <QProgressBar>
@@ -45,17 +49,14 @@
 #include <KDE/KLocale>
 #endif
 
-#ifdef ENABLE_KDE_SUPPORT
-static QString formatNumber(double number, int precision)
+static inline QString formatNumber(double number, int precision)
 {
+    #ifdef ENABLE_KDE_SUPPORT
     return KGlobal::locale()->formatNumber(number, precision);
-}
-#else
-static QString formatNumber(double number, int precision)
-{
+    #else
     return QString::number(number, 'g', precision);
+    #endif
 }
-#endif
 
 enum Columns
 {
@@ -107,6 +108,7 @@ RgDialog::RgDialog(QWidget *parent)
     , state(State_Idle)
     , totalToScan(0)
     , tagReader(0)
+    , autoScanTags(false)
 {
     iCount++;
     setButtons(User1|Ok|Cancel);
@@ -114,14 +116,23 @@ RgDialog::RgDialog(QWidget *parent)
     setAttribute(Qt::WA_DeleteOnClose);
     QWidget *mainWidget = new QWidget(this);
     QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, mainWidget);
+    combo = new QComboBox(this);
     view = new QTreeWidget(this);
     statusLabel = new QLabel(this);
     statusLabel->setVisible(false);
     progress = new QProgressBar(this);
     progress->setVisible(false);
+    combo->addItem(i18n("Show All Tracks"), true);
+    combo->addItem(i18n("Show Untagged Tracks"), false);
     view->setRootIsDecorated(false);
     view->setAllColumnsShowFocus(true);
-    view->setAlternatingRowColors(true);
+    view->setItemDelegate(new BasicItemDelegate(view));
+    view->setAlternatingRowColors(false);
+    view->setContextMenuPolicy(Qt::ActionsContextMenu);
+    view->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    removeAct=new Action(i18n("Remove From List"), view);
+    removeAct->setEnabled(false);
+    view->addAction(removeAct);
     QTreeWidgetItem *hdr = view->headerItem();
     hdr->setText(COL_ARTIST, i18n("Artist"));
     hdr->setText(COL_ALBUM, i18n("Album"));
@@ -142,21 +153,24 @@ RgDialog::RgDialog(QWidget *parent)
     hv->setStretchLastSection(false);
 
     layout->setMargin(0);
+    layout->addWidget(combo);
     layout->addWidget(view);
     layout->addWidget(statusLabel);
     layout->addWidget(progress);
     setMainWidget(mainWidget);
-    #ifdef ENABLE_KDE_SUPPORT
-    setButtonGuiItem(Ok, KStandardGuiItem::save());
-    setButtonGuiItem(Cancel, KStandardGuiItem::close());
-    #else
-    setButtonGuiItem(Ok, GuiItem(i18n("Save"), "document-save"));
-    setButtonGuiItem(Cancel, GuiItem(i18n("Close"), "dialog-close"));
-    #endif
+    setButtonGuiItem(Ok, StdGuiItem::save());
+    setButtonGuiItem(Cancel, StdGuiItem::close());
     setButtonGuiItem(User1, GuiItem(i18n("Scan"), "edit-find"));
     enableButton(Ok, false);
     enableButton(User1, false);
     qRegisterMetaType<Tags::ReplayGain>("Tags::ReplayGain");
+    connect(combo, SIGNAL(currentIndexChanged(int)), SLOT(toggleDisplay()));
+    connect(view, SIGNAL(itemSelectionChanged()), SLOT(controlRemoveAct()));
+    connect(removeAct, SIGNAL(triggered(bool)), SLOT(removeItems()));
+
+    italic=font();
+    italic.setItalic(true);
+    JobController::self()->setMaxActive(1);
 }
 
 RgDialog::~RgDialog()
@@ -165,14 +179,25 @@ RgDialog::~RgDialog()
     iCount--;
 }
 
-void RgDialog::show(const QList<Song> &songs, const QString &udi)
+void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
 {
-    if (songs.count()<1) {
+    if (songs.isEmpty()) {
         deleteLater();
         return;
     }
 
-    origSongs=songs;
+    foreach (const Song &s, songs) {
+        if (!CueFile::isCue(s.file)) {
+           origSongs.append(s);
+        }
+    }
+
+    if (origSongs.isEmpty()) {
+        deleteLater();
+        return;
+    }
+
+    autoScanTags=autoScan;
     qSort(origSongs);
     #ifdef ENABLE_DEVICES_SUPPORT
     if (udi.isEmpty()) {
@@ -194,9 +219,7 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi)
     state=State_Idle;
     enableButton(User1, origSongs.count());
     view->clear();
-    int i=0;
     foreach (const Song &s, origSongs) {
-        albums[createAlbumName(s)].tracks.append(i++);
         new QTreeWidgetItem(view, QStringList() << s.albumArtist() << s.album << s.title);
     }
     Dialog::show();
@@ -211,7 +234,8 @@ void RgDialog::slotButtonClicked(int button)
 
     switch (button) {
     case Ok:
-        if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Update ReplayGain tags in files?"))) {
+        if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Update ReplayGain tags in tracks?"), i18n("Update Tags"),
+                                                       GuiItem(i18n("Update Tags")), StdGuiItem::cancel())) {
             saveTags();
             stopScanning();
             accept();
@@ -223,7 +247,8 @@ void RgDialog::slotButtonClicked(int button)
     case Cancel:
         switch (state) {
         case State_ScanningFiles:
-            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Cancel scanning of files?"))) {
+            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Abort scanning of tracks?"), i18n("Abort"),
+                                                           GuiItem(i18n("Abort")), StdGuiItem::no())) {
                 stopScanning();
                 // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
                 Dialog::slotButtonClicked(button);
@@ -231,7 +256,8 @@ void RgDialog::slotButtonClicked(int button)
             }
             break;
         case State_ScanningTags:
-            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Cancel reading of existing tags?"))) {
+            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Abort reading of existing tags?"), i18n("Abort"),
+                                                           GuiItem(i18n("Abort")), StdGuiItem::no())) {
                 stopReadingTags();
                 // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
                 Dialog::slotButtonClicked(button);
@@ -255,34 +281,38 @@ void RgDialog::slotButtonClicked(int button)
 void RgDialog::startScanning()
 {
     bool all=origTags.isEmpty() ||
-             MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Do you wish to scan all files, or only files without existing tags?"), QString(),
-                                                        GuiItem(i18n("All Tracks")), GuiItem(i18n("Untagged Tracks")));
+             (origTags.count()==origSongs.count()
+                ? MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Scan <b>all</b> tracks?<br><br><i>NOTE: All tracks have existing ReplyGain tags.</i>"), QString(),
+                                                             GuiItem(i18n("Scan")), StdGuiItem::cancel())
+                : MessageBox::No==MessageBox::questionYesNo(this, i18n("Do you wish to scan all tracks, or only tracks without existing tags?"), QString(),
+                                                            GuiItem(i18n("Untagged Tracks")), GuiItem(i18n("All Tracks"))));
     if (!all && origTags.count()==origSongs.count()) {
         return;
     }
-    #ifdef ENABLE_KDE_SUPPORT
-    setButtonGuiItem(Cancel, KStandardGuiItem::cancel());
-    #else
-    setButtonGuiItem(Cancel, GuiItem(i18n("Cancel"), "dialog-cancel"));
-    #endif
+    setButtonGuiItem(Cancel, StdGuiItem::cancel());
     state=State_ScanningFiles;
     enableButton(Ok, false);
     enableButton(User1, false);
+    progress->setValue(0);
     progress->setVisible(true);
-    statusLabel->setText(i18n("Scanning files..."));
+    statusLabel->setText(i18n("Scanning tracks..."));
     statusLabel->setVisible(true);
     clearScanners();
     totalToScan=0;
+    QMap<QString, QList<int> > groupedTracks;
     for (int i=0; i<origSongs.count(); ++i) {
-        if (all || !origTags.contains(i)) {
-            if (scanners.count()<100) {
-                createScanner(i);
-            } else {
-                toScan.append(i);
-            }
-            totalToScan++;
+        if (!removedItems.contains(i) && (all || !origTags.contains(i))) {
+            const Song &sng=origSongs.at(i);
+            groupedTracks[sng.albumArtist()+" -- "+sng.album].append(i);
         }
     }
+    QMap<QString, QList<int> >::ConstIterator it(groupedTracks.constBegin());
+    QMap<QString, QList<int> >::ConstIterator end(groupedTracks.constEnd());
+
+    for (; it!=end; ++it) {
+        createScanner(*it);
+        totalToScan++;
+    }
     progress->setRange(0, 100*totalToScan);
 }
 
@@ -295,40 +325,30 @@ void RgDialog::stopScanning()
 
     JobController::self()->cancel();
     clearScanners();
-    #ifdef ENABLE_KDE_SUPPORT
-    setButtonGuiItem(Cancel, KStandardGuiItem::cancel());
-    #else
-    setButtonGuiItem(Cancel, GuiItem(i18n("Close"), "dialog-close"));
-    #endif
+    setButtonGuiItem(Cancel, StdGuiItem::close());
 }
 
-void RgDialog::createScanner(int index)
+void RgDialog::createScanner(const QList<int> &indexes)
 {
-    Scanner *s=new Scanner(index);
-    s->setFile(base+origSongs.at(index).file);
+    QMap<int, QString> fileMap;
+    foreach (int i, indexes) {
+        fileMap[i]=base+origSongs.at(i).file;
+    }
+
+    AlbumScanner *s=new AlbumScanner(fileMap);
     connect(s, SIGNAL(progress(int)), this, SLOT(scannerProgress(int)));
     connect(s, SIGNAL(done()), this, SLOT(scannerDone()));
-    scanners.insert(index, s);
+    scanners[s]=0;
     JobController::self()->add(s);
 }
 
 void RgDialog::clearScanners()
 {
-    QMap<int, Scanner *>::ConstIterator it(scanners.constBegin());
-    QMap<int, Scanner *>::ConstIterator end(scanners.constEnd());
-
-    for (; it!=end; ++it) {
-        it.value()->stop();
+    QList<AlbumScanner *> scannerList=scanners.keys();
+    foreach (AlbumScanner *sc, scannerList) {
+        sc->stop();
     }
     scanners.clear();
-    toScan.clear();
-    tracks.clear();
-    QMap<QString, Album>::iterator a(albums.begin());
-    QMap<QString, Album>::iterator ae(albums.end());
-
-    for (; a!=ae; ++a) {
-        (*a).scannedTracks.clear();
-    }
 }
 
 void RgDialog::startReadingTags()
@@ -336,11 +356,7 @@ void RgDialog::startReadingTags()
     if (tagReader) {
         return;
     }
-    #ifdef ENABLE_KDE_SUPPORT
-    setButtonGuiItem(Cancel, KStandardGuiItem::cancel());
-    #else
-    setButtonGuiItem(Cancel, GuiItem(i18n("Cancel"), "dialog-cancel"));
-    #endif
+    setButtonGuiItem(Cancel, StdGuiItem::cancel());
     state=State_ScanningTags;
     enableButton(Ok, false);
     enableButton(User1, false);
@@ -369,6 +385,7 @@ void RgDialog::stopReadingTags()
     disconnect(tagReader, SIGNAL(done()), this, SLOT(tagReaderDone()));
     tagReader->requestAbort();
     tagReader=0;
+    autoScanTags=false;
 }
 
 void RgDialog::saveTags()
@@ -380,38 +397,21 @@ void RgDialog::saveTags()
     enableButton(User1, false);
 
     QStringList failed;
-    QMap<QString, Album>::iterator a(albums.begin());
-    QMap<QString, Album>::iterator ae(albums.end());
-    int toSave=0;
-
-    for (; a!=ae; ++a) {
-        foreach (int idx, (*a).tracks) {
-            if (needToSave.contains(idx)) {
-                ++toSave;
-            }
-        }
-    }
 
     progress->setVisible(true);
-    progress->setRange(0, toSave);
+    progress->setRange(0, tagsToSave.count());
 
     int count=0;
-    for (a=albums.begin(); a!=ae; ++a) {
-        foreach (int idx, (*a).tracks) {
-            if (needToSave.contains(idx)) {
-                const Track &track=tracks[idx];
-                if (track.success && Tags::Update_Failed==Tags::updateReplaygain(base+origSongs.at(idx).file,
-                                                                                 Tags::ReplayGain(Scanner::reference(track.data.loudness),
-                                                                                                  Scanner::reference((*a).data.loudness),
-                                                                                                  track.data.peakValue(),
-                                                                                                  (*a).data.peakValue()))) {
-                    failed.append(origSongs.at(idx).file);
-                }
-                progress->setValue(progress->value()+1);
-                if (0==count++%10) {
-                    QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
-                }
-            }
+    QMap<int, Tags::ReplayGain>::ConstIterator it=tagsToSave.constBegin();
+    QMap<int, Tags::ReplayGain>::ConstIterator end=tagsToSave.constEnd();
+
+    for (; it!=end; ++it) {
+        if (Tags::Update_Failed==Tags::updateReplaygain(base+origSongs.at(it.key()).file, it.value())) {
+            failed.append(origSongs.at(it.key()).file);
+        }
+        progress->setValue(progress->value()+1);
+        if (0==count++%10) {
+            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
         }
     }
 
@@ -424,27 +424,22 @@ void RgDialog::updateView()
 {
     int finished=0;
     quint64 totalProgress=0;
-    QMap<int, Track>::iterator it=tracks.begin();
-    QMap<int, Track>::iterator end=tracks.end();
+    QMap<AlbumScanner *, int>::ConstIterator it=scanners.constBegin();
+    QMap<AlbumScanner *, int>::ConstIterator end=scanners.constEnd();
 
     for (; it!=end; ++it) {
-        if ((*it).finished) {
+        if (100==it.value()) {
             finished++;;
         }
-        totalProgress+=(*it).finished ? 100 : (*it).progress;
+        totalProgress+=(*it);
     }
 
     if (finished==totalToScan) {
         progress->setVisible(false);
         statusLabel->setVisible(false);
         state=State_Idle;
-        #ifdef ENABLE_KDE_SUPPORT
-        setButtonGuiItem(Cancel, KStandardGuiItem::close());
-        #else
-        setButtonGuiItem(Cancel, GuiItem(i18n("Close"), "dialog-close"));
-        #endif
-
-        enableButton(Ok, needToSave.count());
+        setButtonGuiItem(Cancel, StdGuiItem::close());
+        enableButton(Ok, !tagsToSave.isEmpty());
     } else {
         progress->setValue(totalProgress);
     }
@@ -475,130 +470,104 @@ Device * RgDialog::getDevice(const QString &udi, QWidget *p)
 
 void RgDialog::scannerProgress(int p)
 {
-    Scanner *s=qobject_cast<Scanner *>(sender());
+    AlbumScanner *s=qobject_cast<AlbumScanner *>(sender());
     if (!s) {
         return;
     }
 
-    tracks[s->index()].progress=p;
+    scanners[s]=p;
     updateView();
 }
 
 void RgDialog::scannerDone()
 {
-    Scanner *s=qobject_cast<Scanner *>(sender());
+    AlbumScanner *s=qobject_cast<AlbumScanner *>(sender());
     if (!s) {
         return;
     }
-    Track &track=tracks[s->index()];
-    if (!track.finished) {
-        track.finished=true;
-        track.success=s->success();
-        QTreeWidgetItem *item=view->topLevelItem(s->index());
-        if (s->success()) {
-            track.progress=100;
-            track.data=s->results();
-            item->setText(COL_TRACKGAIN, i18n("%1 dB").arg(formatNumber(Scanner::reference(track.data.loudness), 2)));
-            item->setText(COL_TRACKPEAK, formatNumber(track.data.peakValue(), 6));
-            QFont f(font());
-            f.setItalic(true);
-            if (origTags.contains(s->index())) {
-                Tags::ReplayGain t=origTags[s->index()];
-                bool diff=false;
-                if (!Utils::equal(t.trackGain, Scanner::reference(track.data.loudness), 0.01)) {
-                    item->setFont(COL_TRACKGAIN, f);
-                    diff=true;
-                }
-                if (!Utils::equal(t.trackPeak, track.data.peakValue(), 0.000001)) {
-                    item->setFont(COL_TRACKPEAK, f);
-                    diff=true;
-                }
-                if (diff) {
-                    item->setFont(COL_ARTIST, f);
-                    item->setFont(COL_TITLE, f);
-                    needToSave.insert(s->index());
-                } else {
-                    needToSave.remove(s->index());
-                }
+
+    const QMap<int, AlbumScanner::Values> &trackValues=s->trackValues();
+    QMap<int, AlbumScanner::Values>::ConstIterator it(trackValues.constBegin());
+    QMap<int, AlbumScanner::Values>::ConstIterator end(trackValues.constEnd());
+
+    if (s->success()) {
+        for(; it!=end; ++it) {
+            Tags::ReplayGain updatedTags(it.value().gain, s->albumValues().gain, it.value().peak, s->albumValues().peak);
+            QTreeWidgetItem *item=view->topLevelItem(it.key());
+            if (it.value().ok) {
+                item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(updatedTags.trackGain, 2)));
+                item->setText(COL_TRACKPEAK, formatNumber(updatedTags.trackPeak, 6));
             } else {
-                item->setFont(COL_ARTIST, f);
-                item->setFont(COL_TITLE, f);
-                item->setFont(COL_TRACKGAIN, f);
-                item->setFont(COL_TRACKPEAK, f);
-                needToSave.insert(s->index());
+                item->setText(COL_TRACKGAIN, i18n("Failed"));
+                item->setText(COL_TRACKPEAK, i18n("Failed"));
             }
-        } else {
+            if (s->albumValues().ok) {
+                item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(updatedTags.albumGain, 2)));
+                item->setText(COL_ALBUMPEAK, formatNumber(updatedTags.albumPeak, 6));
+            } else {
+                item->setText(COL_ALBUMGAIN, i18n("Failed"));
+                item->setText(COL_ALBUMPEAK, i18n("Failed"));
+            }
+
+            if (it.value().ok && origTags.contains(it.key())) {
+                Tags::ReplayGain t=origTags[it.key()];
+                bool diff=false;
+                bool diffAlbum=false;
+                if (!Utils::equal(t.trackGain, updatedTags.trackGain, 0.01)) {
+                    item->setFont(COL_TRACKGAIN, italic);
+                    diff=true;
+                }
+                if (!Utils::equal(t.trackPeak, updatedTags.trackPeak, 0.000001)) {
+                    item->setFont(COL_TRACKPEAK, italic);
+                    diff=true;
+                }
+                if (!Utils::equal(t.albumGain, updatedTags.albumGain, 0.01)) {
+                    item->setFont(COL_ALBUMGAIN, italic);
+                    diffAlbum=true;
+                }
+                if (!Utils::equal(t.albumPeak, updatedTags.albumPeak, 0.000001)) {
+                    item->setFont(COL_ALBUMPEAK, italic);
+                    diffAlbum=true;
+                }
+                if (diff || diffAlbum) {
+                    if (diff) {
+                        item->setFont(COL_ARTIST, italic);
+                        item->setFont(COL_TITLE, italic);
+                    }
+                    if (diffAlbum) {
+                        item->setFont(COL_ALBUM, italic);
+                    }
+                    tagsToSave.insert(it.key(), updatedTags);
+                } else {
+                    tagsToSave.remove(it.key());
+                }
+            } else if (it.value().ok) {
+                item->setFont(COL_ARTIST, italic);
+                item->setFont(COL_ALBUM, italic);
+                item->setFont(COL_TITLE, italic);
+                item->setFont(COL_TRACKGAIN, italic);
+                item->setFont(COL_TRACKPEAK, italic);
+                item->setFont(COL_ALBUMGAIN, italic);
+                item->setFont(COL_ALBUMPEAK, italic);
+                tagsToSave.insert(it.key(), updatedTags);
+            } else {
+                tagsToSave.remove(it.key());
+            }
+        }
+    } else {
+        for(; it!=end; ++it) {
+            QTreeWidgetItem *item=view->topLevelItem(it.key());
             item->setText(COL_TRACKGAIN, i18n("Failed"));
             item->setText(COL_TRACKPEAK, i18n("Failed"));
-            needToSave.remove(s->index());
+            item->setText(COL_ALBUMGAIN, i18n("Failed"));
+            item->setText(COL_ALBUMPEAK, i18n("Failed"));
+            tagsToSave.remove(it.key());
         }
-
-        QMap<QString, Album>::iterator a(albums.find(createAlbumName(origSongs.at(s->index()))));
-        QMap<QString, Album>::iterator ae(albums.end());
-
-        if (a!=ae) {
-            (*a).scannedTracks.insert(s->index());
-            if ((*a).scannedTracks.count()==(*a).tracks.count()) {
-                QList<Scanner *> as;
-                foreach (int idx, (*a).tracks) {
-                    Scanner *s=scanners[idx];
-                    if (s && s->success()) {
-                        as.append(s);
-                    }
-                }
-
-                if (as.count()) {
-                    QFont f(font());
-                    f.setItalic(true);
-                    (*a).data=Scanner::global(as);
-                    foreach (int idx, (*a).tracks) {
-                        QTreeWidgetItem *item=view->topLevelItem(idx);
-                        item->setText(COL_ALBUMGAIN, i18n("%1 dB").arg(formatNumber(Scanner::reference((*a).data.loudness), 2)));
-                        item->setText(COL_ALBUMPEAK, formatNumber((*a).data.peak, 6));
-
-                        if (origTags.contains(idx)) {
-                            Tags::ReplayGain t=origTags[idx];
-                            bool diff=false;
-                            if (!Utils::equal(t.albumGain, Scanner::reference((*a).data.loudness), 0.01)) {
-                                item->setFont(COL_ALBUMGAIN, f);
-                                needToSave.insert(idx);
-                                diff=true;
-                            }
-                            if (!Utils::equal(t.albumPeak, (*a).data.peak, 0.000001)) {
-                                item->setFont(COL_ALBUMPEAK, f);
-                                needToSave.insert(idx);
-                                diff=true;
-                            }
-
-                            if (diff) {
-                                item->setFont(COL_ARTIST, f);
-                                item->setFont(COL_ALBUM, f);
-                            }
-                        } else {
-                            item->setFont(COL_ARTIST, f);
-                            item->setFont(COL_ALBUM, f);
-                            item->setFont(COL_ALBUMGAIN, f);
-                            item->setFont(COL_ALBUMPEAK, f);
-                        }
-                    }
-                }
-
-                foreach (int idx, (*a).tracks) {
-                    Scanner *s=scanners[idx];
-                    if (s) {
-                        scanners.remove(idx);
-                        JobController::self()->finishedWith(s);
-                    }
-                }
-            }
-        }
-        updateView();
     }
 
-    if (!toScan.isEmpty()) {
-        int index=toScan.takeAt(0);
-        createScanner(index);
-    }
+    scanners[s]=100;
+    updateView();
+    JobController::self()->finishedWith(s);
 }
 
 void RgDialog::songTags(int index, Tags::ReplayGain tags)
@@ -612,9 +581,9 @@ void RgDialog::songTags(int index, Tags::ReplayGain tags)
             if (!item) {
                 return;
             }
-            item->setText(COL_TRACKGAIN, i18n("%1 dB").arg(formatNumber(tags.trackGain, 2)));
+            item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(tags.trackGain, 2)));
             item->setText(COL_TRACKPEAK, formatNumber(tags.trackPeak, 6));
-            item->setText(COL_ALBUMGAIN, i18n("%1 dB").arg(formatNumber(tags.albumGain, 2)));
+            item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(tags.albumGain, 2)));
             item->setText(COL_ALBUMPEAK, formatNumber(tags.albumPeak, 6));
         }
     }
@@ -634,6 +603,48 @@ void RgDialog::tagReaderDone()
     enableButton(User1, true);
     progress->setVisible(false);
     statusLabel->setVisible(false);
+
+    if (autoScanTags) {
+        autoScanTags=false;
+        startScanning();
+    }
+}
+
+void RgDialog::toggleDisplay()
+{
+    bool showAll=combo->itemData(combo->currentIndex()).toBool();
+
+    for (int i=0; i<view->topLevelItemCount(); ++i) {
+        QTreeWidgetItem *item=view->topLevelItem(i);
+        if (!removedItems.contains(view->indexOfTopLevelItem(item))) {
+            view->setItemHidden(item, showAll ? false : origTags.contains(i));
+        }
+    }
+}
+
+void RgDialog::controlRemoveAct()
+{
+    removeAct->setEnabled(view->topLevelItemCount()>1 && !view->selectedItems().isEmpty());
+}
+
+void RgDialog::removeItems()
+{
+    if (view->topLevelItemCount()<1) {
+        return;
+    }
+
+    if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Remove the selected tracks from the list?"),
+                                                   i18n("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
+        QList<QTreeWidgetItem *> selection=view->selectedItems();
+        foreach (QTreeWidgetItem *item, selection) {
+            int index=view->indexOfTopLevelItem(item);
+            view->setItemHidden(item, true);
+            removedItems.insert(index);
+            if (tagsToSave.contains(index)) {
+                tagsToSave.remove(index);
+            }
+        }
+    }
 }
 
 void RgDialog::closeEvent(QCloseEvent *event)
