@@ -27,7 +27,7 @@
 #include <QSet>
 #include "treeview.h"
 
-class Song;
+struct Song;
 
 class GroupedView : public TreeView
 {
@@ -81,7 +81,6 @@ public:
     QModelIndexList selectedIndexes() const;
     void dropEvent(QDropEvent *event);
     void collectionRemoved(quint32 key);
-    void expandAll();
     void expand(const QModelIndex &idx);
 
 public Q_SLOTS:
