@@ -33,6 +33,7 @@
 #include "config.h"
 #include "gtkstyle.h"
 #include "spinner.h"
+#include "messageoverlay.h"
 #include <QToolButton>
 #include <QStyle>
 #include <QStyleOptionViewItem>
@@ -43,14 +44,6 @@
 #include <KDE/KGlobalSettings>
 #endif
 
-static bool forceSingleClick=true;
-
-#ifdef ENABLE_KDE_SUPPORT
-#define SINGLE_CLICK KGlobalSettings::singleClick()
-#else
-#define SINGLE_CLICK style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this)
-#endif
-
 static int listDecorationSize=22;
 static int treeDecorationSize=16;
 
@@ -91,23 +84,6 @@ bool ViewEventHandler::eventFilter(QObject *obj, QEvent *event)
     return QObject::eventFilter(obj, event);
 }
 
-ListViewEventHandler::ListViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v, QAction *a)
-    : ViewEventHandler(d, v)
-    , act(a)
-{
-}
-
-bool ListViewEventHandler::eventFilter(QObject *obj, QEvent *event)
-{
-    if (view->hasFocus() && QEvent::KeyRelease==event->type() && static_cast<QKeyEvent *>(event)->key()==Qt::Key_Escape) {
-        if (act->isEnabled()) {
-            act->trigger();
-        }
-        return true;
-    }
-    return ViewEventHandler::eventFilter(obj, event);
-}
-
 static const int constImageSize=22;
 static const int constDevImageSize=32;
 
@@ -207,6 +183,10 @@ public:
         ActionPos actionPos = iconMode ? AP_VTop : AP_HMiddle;
         bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
 
+        if (childText==QLatin1String("-")) {
+            childText.clear();
+        }
+
         painter->save();
         painter->setClipRect(r);
 
@@ -269,9 +249,9 @@ public:
             }
         }
 
-        if (!mouseOver) {
-            drawIcons(painter, AP_VTop==actionPos ? r2 : r, false, rtl, actionPos, index);
-        }
+//        if (!mouseOver) {
+//            drawIcons(painter, AP_VTop==actionPos ? r2 : r, false, rtl, actionPos, index);
+//        }
 
         QRect textRect;
         QColor color(option.palette.color(active ? QPalette::Active : QPalette::Inactive, selected ? QPalette::HighlightedText : QPalette::Text));
@@ -300,11 +280,13 @@ public:
             painter->setFont(textFont);
             painter->drawText(textRect, text, textOpt);
 
-            childText = childMetrics.elidedText(childText, Qt::ElideRight, childRect.width(), QPalette::WindowText);
-            color.setAlphaF(subTextAlpha(selected));
-            painter->setPen(color);
-            painter->setFont(childFont);
-            painter->drawText(childRect, childText, textOpt);
+            if (!childText.isEmpty()) {
+                childText = childMetrics.elidedText(childText, Qt::ElideRight, childRect.width(), QPalette::WindowText);
+                color.setAlphaF(subTextAlpha(selected));
+                painter->setPen(color);
+                painter->setFont(childFont);
+                painter->drawText(childRect, childText, textOpt);
+            }
         }
 
         if (showCapacity) {
@@ -333,7 +315,7 @@ public:
         if (drawBgnd && mouseOver) {
             drawIcons(painter, AP_VTop==actionPos ? r2 : r, true, rtl, actionPos, index);
         }
-
+        drawDivider(painter, option.rect, color);
         painter->restore();
     }
 
@@ -379,12 +361,11 @@ public:
         QStringList text=index.data(Qt::DisplayRole).toString().split("\n");
         bool gtk=GtkStyle::isActive();
         bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
-
+        bool selected=option.state&QStyle::State_Selected;
+        bool active=option.state&QStyle::State_Active;
         if (!gtk && 1==text.count()) {
             QStyledItemDelegate::paint(painter, option, index);
         } else {
-            bool selected=option.state&QStyle::State_Selected;
-            bool active=option.state&QStyle::State_Active;
             if ((option.state&QStyle::State_MouseOver) && gtk) {
                 GtkStyle::drawSelection(option, painter, selected ? 0.75 : 0.25);
             } else {
@@ -443,6 +424,8 @@ public:
         if ((option.state & QStyle::State_MouseOver)) {
             drawIcons(painter, option.rect, true, rtl, AP_HMiddle, index);
         }
+        drawDivider(painter, option.rect, option.palette.color(active ? QPalette::Active : QPalette::Inactive,
+                                                               selected ? QPalette::HighlightedText : QPalette::Text));
     }
 
     void setSimple(bool s) { simpleStyle=s; }
@@ -481,16 +464,6 @@ QString ItemView::modeStr(Mode m)
     }
 }
 
-void ItemView::setForceSingleClick(bool v)
-{
-    forceSingleClick=v;
-}
-
-bool ItemView::getForceSingleClick()
-{
-    return forceSingleClick;
-}
-
 ItemView::ItemView(QWidget *p)
     : QWidget(p)
     , searchTimer(0)
@@ -499,15 +472,21 @@ ItemView::ItemView(QWidget *p)
     , mode(Mode_SimpleTree)
     , groupedView(0)
     , spinner(0)
+    , msgOverlay(0)
 {
     setupUi(this);
-    backAction = new QAction(i18n("Back"), this);
+    backAction = new QAction(i18n("Go Back"), this);
     backAction->setIcon(Icon("go-previous"));
     backButton->setDefaultAction(backAction);
-    //backButton->setMaximumHeight(listSearch->sizeHint().height());
-    backAction->setShortcut(Qt::Key_Escape);
+    backAction->setShortcut(Qt::AltModifier+(Qt::LeftToRight==layoutDirection() ? Qt::Key_Left : Qt::Key_Right));
+    homeAction = new QAction(i18n("Go Home"), this);
+    homeAction->setIcon(Icon("go-home"));
+    homeButton->setDefaultAction(homeAction);
+    homeAction->setShortcut(Qt::AltModifier+Qt::Key_Up);
     listView->addAction(backAction);
+    listView->addAction(homeAction);
     listView->addDefaultAction(backAction);
+    listView->addDefaultAction(homeAction);
     QAction *sep=new QAction(this);
     sep->setSeparator(true);
     listView->addAction(sep);
@@ -520,23 +499,22 @@ ItemView::ItemView(QWidget *p)
     TreeDelegate *td=new TreeDelegate(treeView);
     listView->setItemDelegate(ld);
     treeView->setItemDelegate(td);
-    listView->installEventFilter(new ListViewEventHandler(ld, listView, backAction));
+    listView->installEventFilter(new ViewEventHandler(ld, listView));
     treeView->installEventFilter(new ViewEventHandler(td, treeView));
-    connect(treeSearch, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
-    connect(treeSearch, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
-    connect(listSearch, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
-    connect(listSearch, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
+    connect(searchWidget, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
+    connect(searchWidget, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
+    connect(searchWidget, SIGNAL(active(bool)), this, SIGNAL(searchIsActive(bool)));
     connect(treeView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
-    if (SINGLE_CLICK) {
-        connect(treeView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
-    }
+    connect(treeView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
     connect(treeView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
     connect(treeView, SIGNAL(clicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
     connect(listView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
-    connect(listView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
+    connect(listView, SIGNAL(activated(const QModelIndex &)), this, SLOT(activateItem(const QModelIndex &)));
     connect(listView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
     connect(listView, SIGNAL(clicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
     connect(backAction, SIGNAL(triggered(bool)), this, SLOT(backActivated()));
+    connect(homeAction, SIGNAL(triggered(bool)), this, SLOT(homeActivated()));
+    searchWidget->setVisible(false);
 }
 
 ItemView::~ItemView()
@@ -554,9 +532,7 @@ void ItemView::allowGroupedView()
         groupedView->setMultiLevel(true);
         treeLayout->addWidget(groupedView);
         connect(groupedView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
-        if (SINGLE_CLICK) {
-            connect(groupedView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
-        }
+        connect(groupedView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
         connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
         connect(groupedView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
     }
@@ -581,9 +557,13 @@ void ItemView::setMode(Mode m)
         return;
     }
 
+    QIcon oldBgndIcon=bgndIcon;
+    if (!bgndIcon.isNull()) {
+        setBackgroundImage(QIcon());
+    }
+
     mode=m;
-    treeSearch->setText(QString());
-    listSearch->setText(QString());
+    searchWidget->setText(QString());
     if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         treeView->setModel(itemModel);
         listView->setModel(0);
@@ -616,7 +596,7 @@ void ItemView::setMode(Mode m)
             listView->setGridSize(listGridSize);
             listView->setViewMode(QListView::ListMode);
             listView->setResizeMode(QListView::Fixed);
-            listView->setAlternatingRowColors(true);
+//            listView->setAlternatingRowColors(true);
             listView->setWordWrap(false);
         }
     }
@@ -628,16 +608,13 @@ void ItemView::setMode(Mode m)
             spinner->start();
         }
     }
-}
+    if (msgOverlay) {
+        msgOverlay->setWidget(view()->viewport());
+    }
 
-void ItemView::hideBackButton()
-{
-    backButton->setVisible(false);
-    listLayout->removeWidget(backButton);
-    listLayout->removeWidget(listSearch);
-    listLayout->removeWidget(listView);
-    listLayout->addWidget(listSearch, 0, 0, 1, 1);
-    listLayout->addWidget(listView, 1, 0, 1, 1);
+    if (!oldBgndIcon.isNull()) {
+        setBackgroundImage(oldBgndIcon);
+    }
 }
 
 QModelIndexList ItemView::selectedIndexes() const
@@ -655,6 +632,8 @@ void ItemView::setLevel(int l, bool haveChildren)
     int prev=currentLevel;
     currentLevel=l;
     backAction->setEnabled(0!=l);
+    homeAction->setEnabled(l>1);
+    homeAction->setVisible(l>1);
 
     if (Mode_IconTop==mode) {
         if (0==currentLevel || haveChildren) {
@@ -662,17 +641,19 @@ void ItemView::setLevel(int l, bool haveChildren)
                 listView->setGridSize(iconGridSize);
                 listView->setViewMode(QListView::IconMode);
                 listView->setResizeMode(QListView::Adjust);
-                listView->setAlternatingRowColors(false);
+//                listView->setAlternatingRowColors(false);
                 listView->setWordWrap(true);
                 listView->setDragDropMode(QAbstractItemView::DragOnly);
+                ((ActionItemDelegate *)listView->itemDelegate())->setLargeIcons(iconGridSize.width()>(ActionItemDelegate::constLargeActionIconSize*6));
             }
         } else if(QListView::ListMode!=listView->viewMode()) {
             listView->setGridSize(listGridSize);
             listView->setViewMode(QListView::ListMode);
             listView->setResizeMode(QListView::Fixed);
-            listView->setAlternatingRowColors(true);
+//            listView->setAlternatingRowColors(true);
             listView->setWordWrap(false);
             listView->setDragDropMode(QAbstractItemView::DragOnly);
+            ((ActionItemDelegate *)listView->itemDelegate())->setLargeIcons(false);
         }
     }
 
@@ -680,12 +661,16 @@ void ItemView::setLevel(int l, bool haveChildren)
         view()->selectionModel()->clearSelection();
     }
 
-    if (0==currentLevel) {
-        listSearch->setPlaceholderText(i18n("Search %1...").arg(topText));
-    } else if (prev>currentLevel) {
-        listSearch->setPlaceholderText(prevText[currentLevel]);
-    } else {
-        prevText.insert(prev, listSearch->placeholderText());
+    backButton->setVisible(currentLevel>0);
+    title->setVisible(currentLevel>0);
+    homeButton->setVisible(currentLevel>1);
+
+    if (currentLevel>0) {
+        if (prev>currentLevel) {
+            title->setText(prevText[currentLevel]);
+        } else {
+            prevText.insert(prev, title->text());
+        }
     }
 }
 
@@ -713,21 +698,12 @@ void ItemView::setModel(ProxyModel *m)
 
 QString ItemView::searchText() const
 {
-    return Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode ? treeSearch->text() : listSearch->text();
+    return searchWidget->isVisible() ? searchWidget->text() : QString();
 }
 
 void ItemView::clearSearchText()
 {
-    return Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode ? treeSearch->setText(QString()) : listSearch->setText(QString());
-}
-
-void ItemView::setTopText(const QString &text)
-{
-    topText=text;
-    if (0==currentLevel) {
-        listSearch->setPlaceholderText(i18n("Search %1...").arg(topText));
-    }
-    treeSearch->setPlaceholderText(i18n("Search %1...").arg(topText));
+    return searchWidget->setText(QString());
 }
 
 void ItemView::setUniformRowHeights(bool v)
@@ -767,6 +743,7 @@ void ItemView::setGridSize(const QSize &sz)
     iconGridSize=sz;
     if (Mode_IconTop==mode && 0==currentLevel) {
         listView->setGridSize(listGridSize);
+        ((ActionItemDelegate *)listView->itemDelegate())->setLargeIcons(iconGridSize.width()>(ActionItemDelegate::constLargeActionIconSize*6));
     }
 }
 
@@ -839,11 +816,22 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
 
 void ItemView::focusSearch()
 {
-    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode) {
-        treeSearch->setFocus();
-    } else {
-        listSearch->setFocus();
-    }
+    searchWidget->activate();
+}
+
+void ItemView::setSearchLabelText(const QString &text)
+{
+    searchWidget->setLabel(text);
+}
+
+void ItemView::setSearchVisible(bool v)
+{
+    searchWidget->setVisible(v);
+}
+
+bool ItemView::isSearchActive() const
+{
+    return searchWidget->isActive();
 }
 
 void ItemView::setStartClosed(bool sc)
@@ -872,22 +860,56 @@ void ItemView::updateRows(const QModelIndex &idx)
     }
 }
 
-void ItemView::expandAll()
+void ItemView::expandAll(const QModelIndex &index)
 {
     if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
-        treeView->expandAll();
+        treeView->expandAll(index);
     } else if (Mode_GroupedTree==mode && groupedView) {
-        groupedView->expandAll();
+        groupedView->expandAll(index);
     }
 }
 
-void ItemView::showSpinner()
+void ItemView::expand(const QModelIndex &index)
 {
-    if (!spinner) {
-        spinner=new Spinner(this);
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
+        treeView->expand(index);
+    } else if (Mode_GroupedTree==mode && groupedView) {
+        groupedView->expand(index);
+    }
+}
+
+void ItemView::showMessage(const QString &message, int timeout)
+{
+    if (!msgOverlay) {
+        msgOverlay=new MessageOverlay(this);
+        msgOverlay->setWidget(view()->viewport());
+    }
+    msgOverlay->setText(message, timeout, timeout<=0);
+}
+
+void ItemView::setBackgroundImage(const QIcon &icon)
+{
+    bgndIcon=icon;
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
+        treeView->setBackgroundImage(bgndIcon);
+    } else if (Mode_GroupedTree==mode && groupedView) {
+        groupedView->setBackgroundImage(bgndIcon);
+    } else if (Mode_List==mode || Mode_IconTop==mode) {
+        listView->setBackgroundImage(bgndIcon);
+    }
+}
+
+void ItemView::showSpinner(bool v)
+{
+    if (v) {
+        if (!spinner) {
+            spinner=new Spinner(this);
+        }
+        spinner->setWidget(view()->viewport());
+        spinner->start();
+    } else {
+        hideSpinner();
     }
-    spinner->setWidget(view()->viewport());
-    spinner->start();
 }
 
 void ItemView::hideSpinner()
@@ -924,6 +946,17 @@ void ItemView::backActivated()
     }
 }
 
+void ItemView::homeActivated()
+{
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode || 0==currentLevel) {
+        return;
+    }
+    setLevel(0);
+    itemModel->setRootIndex(QModelIndex());
+    listView->setRootIndex(QModelIndex());
+    emit rootIndexSet(QModelIndex());
+}
+
 void ItemView::setExpanded(const QModelIndex &idx, bool exp)
 {
     if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
@@ -946,14 +979,14 @@ void ItemView::itemClicked(const QModelIndex &index)
         return;
     }
 
-    if (forceSingleClick) {
+    if (TreeView::getForceSingleClick()) {
         activateItem(index);
     }
 }
 
 void ItemView::itemActivated(const QModelIndex &index)
 {
-    if (!forceSingleClick) {
+    if (!TreeView::getForceSingleClick()) {
         activateItem(index);
     }
 }
@@ -970,17 +1003,17 @@ void ItemView::activateItem(const QModelIndex &index, bool emitRootSet)
         if (!index.parent().isValid()) {
             groupedView->setExpanded(index, !groupedView->TreeView::isExpanded(index));
         }
-    } else if (index.isValid() && index.child(0, 0).isValid()) {
+    } else if (index.isValid() && index.child(0, 0).isValid() && index!=listView->rootIndex()) {
         prevTopIndex=listView->indexAt(QPoint(8, 8));
         if (qobject_cast<QSortFilterProxyModel *>(listView->model())) {
             prevTopIndex=static_cast<QSortFilterProxyModel *>(listView->model())->mapToSource(prevTopIndex);
         }
         setLevel(currentLevel+1, index.child(0, 0).child(0, 0).isValid());
-        QString text=index.data(Role_Search).toString();
+        QString text=index.data(Role_TitleText).toString();
         if (text.isEmpty()) {
             text=index.data(Qt::DisplayRole).toString();
         }
-        listSearch->setPlaceholderText(i18n("Search %1...").arg(text));
+        title->setText(text);
         listView->setRootIndex(index);
         itemModel->setRootIndex(index);
         if (emitRootSet) {
@@ -992,8 +1025,7 @@ void ItemView::activateItem(const QModelIndex &index, bool emitRootSet)
 
 void ItemView::delaySearchItems()
 {
-    bool isTree=Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode;
-    if ((isTree && treeSearch->text().isEmpty()) || (!isTree && listSearch->text().isEmpty())) {
+    if (searchWidget->text().isEmpty()) {
         if (searchTimer) {
             searchTimer->stop();
         }
