@@ -30,13 +30,18 @@
 #include "mpdstatus.h"
 #include "localize.h"
 #include "spinner.h"
+#include "messageoverlay.h"
 #include <QHeaderView>
 #include <QMenu>
 #include <QAction>
 #include <QFile>
+#include <QPainter>
+#include <QApplication>
+#include <qglobal.h>
 
-PlayQueueTreeView::PlayQueueTreeView(QWidget *parent)
+PlayQueueTreeView::PlayQueueTreeView(PlayQueueView *parent)
     : TreeView(parent, true)
+    , view(parent)
     , menu(0)
 {
     setContextMenuPolicy(Qt::CustomContextMenu);
@@ -50,12 +55,35 @@ PlayQueueTreeView::PlayQueueTreeView(QWidget *parent)
     setDropIndicatorShown(true);
     setRootIsDecorated(false);
     setUniformRowHeights(true);
+    setUseSimpleDelegate();
 }
 
 PlayQueueTreeView::~PlayQueueTreeView()
 {
 }
 
+void PlayQueueTreeView::paintEvent(QPaintEvent *e)
+{
+    view->drawBackdrop(viewport(), size());
+    TreeView::paintEvent(e);
+}
+
+PlayQueueGroupedView::PlayQueueGroupedView(PlayQueueView *parent)
+    : GroupedView(parent, true)
+    , view(parent)
+{
+}
+
+PlayQueueGroupedView::~PlayQueueGroupedView()
+{
+}
+
+void PlayQueueGroupedView::paintEvent(QPaintEvent *e)
+{
+    view->drawBackdrop(viewport(), size());
+    GroupedView::paintEvent(e);
+}
+
 #if QT_VERSION < 0x050000
 static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
 {
@@ -84,12 +112,18 @@ void PlayQueueTreeView::initHeader()
         return;
     }
 
-    QFontMetrics fm(font());
     QHeaderView *hdr=header();
     if (!menu) {
+        QFont f(font());
+        f.setBold(true);
+        QFontMetrics fm(f);
         setResizeMode(hdr, QHeaderView::Interactive);
         hdr->setContextMenuPolicy(Qt::CustomContextMenu);
-        hdr->resizeSection(PlayQueueModel::COL_STATUS, 20);
+        int statusSize=model()->data(QModelIndex(), Qt::SizeHintRole).toSize().width();
+        if (statusSize<20) {
+            statusSize=20;
+        }
+        hdr->resizeSection(PlayQueueModel::COL_STATUS, statusSize);
         hdr->resizeSection(PlayQueueModel::COL_TRACK, fm.width("999"));
         hdr->resizeSection(PlayQueueModel::COL_YEAR, fm.width("99999"));
         setResizeMode(hdr, PlayQueueModel::COL_STATUS, QHeaderView::Fixed);
@@ -176,8 +210,10 @@ void PlayQueueTreeView::toggleHeaderItem(bool visible)
 PlayQueueView::PlayQueueView(QWidget *parent)
     : QStackedWidget(parent)
     , spinner(0)
+    , msgOverlay(0)
+    , useCoverAsBgnd(false)
 {
-    groupedView=new GroupedView(this, true);
+    groupedView=new PlayQueueGroupedView(this);
     groupedView->setIndentation(0);
     groupedView->setItemsExpandable(false);
     groupedView->setExpandsOnDoubleClick(false);
@@ -190,6 +226,9 @@ PlayQueueView::PlayQueueView(QWidget *parent)
     connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), SIGNAL(doubleClicked(const QModelIndex &)));
     connect(treeView, SIGNAL(doubleClicked(const QModelIndex &)), SIGNAL(doubleClicked(const QModelIndex &)));
     setContextMenuPolicy(Qt::ActionsContextMenu);
+
+    animator.setPropertyName("fade");
+    animator.setTargetObject(this);
 }
 
 PlayQueueView::~PlayQueueView()
@@ -225,6 +264,9 @@ void PlayQueueView::setGrouped(bool g)
                 spinner->start();
             }
         }
+        if (msgOverlay) {
+            msgOverlay->setWidget(view()->viewport());
+        }
     }
 }
 
@@ -365,3 +407,100 @@ void PlayQueueView::hideSpinner()
         spinner->stop();
     }
 }
+
+void PlayQueueView::setFade(float value)
+{
+    if (fadeValue!=value) {
+        fadeValue = value;
+        if (qFuzzyCompare(fadeValue, qreal(1.0))) {
+            previousBackground=QPixmap();
+        }
+        view()->viewport()->update();
+    }
+}
+
+void PlayQueueView::setUseCoverAsBackgrond(bool u)
+{
+    if (u==useCoverAsBgnd) {
+        return;
+    }
+    useCoverAsBgnd=u;
+    updatePalette();
+
+    if (!u) {
+        previousBackground=QPixmap();
+        curentCover=QImage();
+        curentBackground=QPixmap();
+        view()->viewport()->update();
+    }
+}
+
+void PlayQueueView::updatePalette()
+{
+    QPalette pal=palette();
+
+    if (useCoverAsBgnd) {
+        pal.setColor(QPalette::Base, Qt::transparent);
+    }
+
+    groupedView->setPalette(pal);
+    groupedView->viewport()->setPalette(pal);
+    treeView->setPalette(pal);
+    treeView->viewport()->setPalette(pal);
+}
+
+void PlayQueueView::setImage(const QImage &img)
+{
+    if (!useCoverAsBgnd) {
+        return;
+    }
+    previousBackground=curentBackground;
+    curentCover=img.isNull() ? QImage() : TreeView::setOpacity(img);
+    curentBackground=QPixmap();
+    animator.stop();
+    if (isVisible()) {
+        fadeValue=0.0;
+        animator.setDuration(250);
+        animator.setEndValue(1.0);
+        animator.start();
+    }
+}
+
+void PlayQueueView::streamFetchStatus(const QString &msg)
+{
+    if (!msgOverlay) {
+        msgOverlay=new MessageOverlay(this);
+        msgOverlay->setWidget(view()->viewport());
+        connect(msgOverlay, SIGNAL(cancel()), SIGNAL(cancelStreamFetch()));
+        connect(msgOverlay, SIGNAL(cancel()), SLOT(hideSpinner()));
+    }
+    msgOverlay->setText(msg);
+}
+
+void PlayQueueView::drawBackdrop(QWidget *widget, const QSize &size)
+{
+    if (!useCoverAsBgnd) {
+        return;
+    }
+
+    QPainter p(widget);
+
+    p.fillRect(0, 0, size.width(), size.height(), QApplication::palette().color(QPalette::Base));
+    if (!curentCover.isNull() || !previousBackground.isNull()) {
+        if (!curentCover.isNull() && (size!=lastBgndSize || curentBackground.isNull())) {
+            curentBackground = QPixmap::fromImage(curentCover.scaled(size, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
+            lastBgndSize=size;
+        }
+
+        if (!previousBackground.isNull()) {
+            if (!qFuzzyCompare(fadeValue, qreal(0.0))) {
+                p.setOpacity(1.0-fadeValue);
+            }
+            p.drawPixmap((size.width()-previousBackground.width())/2, (size.height()-previousBackground.height())/2, previousBackground);
+        }
+        if (!curentBackground.isNull()) {
+            p.setOpacity(fadeValue);
+            p.drawPixmap((size.width()-curentBackground.width())/2, (size.height()-curentBackground.height())/2, curentBackground);
+        }
+    }
+}
