@@ -25,12 +25,66 @@
 #include "itemview.h"
 #include "icons.h"
 #include "config.h"
+#include "basicitemdelegate.h"
 #include <QMouseEvent>
 #include <QPaintEvent>
 #include <QPainter>
 #include <QDrag>
 #include <QMimeData>
 #include <QMap>
+#include <QStyle>
+
+#ifdef ENABLE_KDE_SUPPORT
+#include <KDE/KGlobalSettings>
+#endif
+
+#ifdef ENABLE_KDE_SUPPORT
+#define SINGLE_CLICK KGlobalSettings::singleClick()
+#else
+#define SINGLE_CLICK style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this)
+#endif
+
+
+QImage TreeView::setOpacity(const QImage &orig, double opacity)
+{
+    QImage img=QImage::Format_ARGB32==orig.format() ? orig : orig.convertToFormat(QImage::Format_ARGB32);
+    uchar *bits = img.bits();
+    for (int i = 0; i < img.height()*img.bytesPerLine(); i+=4) {
+        if (0!=bits[i+3]) {
+            bits[i+3] = opacity*255;
+        }
+    }
+    return img;
+}
+
+QPixmap TreeView::createBgndPixmap(const QIcon &icon)
+{
+    if (icon.isNull()) {
+        return QPixmap();
+    }
+    static int bgndSize=0;
+    if (0==bgndSize) {
+        bgndSize=QApplication::fontMetrics().height()*16;
+    }
+
+    QImage img=icon.pixmap(bgndSize, bgndSize).toImage();
+    if (img.width()!=bgndSize && img.height()!=bgndSize) {
+        img=img.scaled(bgndSize, bgndSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    }
+    return QPixmap::fromImage(setOpacity(img, 0.075));
+}
+
+static bool forceSingleClick=true;
+
+void TreeView::setForceSingleClick(bool v)
+{
+    forceSingleClick=v;
+}
+
+bool TreeView::getForceSingleClick()
+{
+    return forceSingleClick;
+}
 
 TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
         : QTreeView(parent)
@@ -40,7 +94,7 @@ TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
     setContextMenuPolicy(Qt::NoContextMenu);
 //     setRootIsDecorated(false);
     setAllColumnsShowFocus(true);
-    setAlternatingRowColors(true);
+    setAlternatingRowColors(false);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     setSelectionBehavior(QAbstractItemView::SelectRows);
     // Treeview does not seem to need WA_MouseTracking set, even with QGtkStyle items still
@@ -48,6 +102,10 @@ TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
     // errors if an item is dragged onto playqueue whilst playqueue has a selected item!
     // BUG:145
     //setAttribute(Qt::WA_MouseTracking);
+
+    if (SINGLE_CLICK) {
+        connect(this, SIGNAL(activated(const QModelIndex &)), this, SIGNAL(itemActivated(const QModelIndex &)));
+    }
 }
 
 TreeView::~TreeView()
@@ -63,6 +121,11 @@ void TreeView::setPageDefaults()
     setAnimated(true);
 }
 
+void TreeView::setExpandOnClick()
+{
+    connect(this, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemWasClicked(const QModelIndex &)));
+}
+
 void TreeView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
 {
     QTreeView::selectionChanged(selected, deselected);
@@ -108,7 +171,7 @@ void TreeView::startDrag(Qt::DropActions supportedActions)
             }
         }
         if (pix.isNull()) {
-            drag->setPixmap(Icons::albumIcon.pixmap(32, 32));
+            drag->setPixmap(Icons::self()->albumIcon.pixmap(32, 32));
         } else {
             drag->setPixmap(pix.width()<32 ? pix : pix.scaled(QSize(32, 32), Qt::KeepAspectRatio, Qt::SmoothTransformation));
         }
@@ -134,11 +197,11 @@ QModelIndexList TreeView::selectedIndexes() const
     return sort.values();
 }
 
-void TreeView::expandAll()
+void TreeView::expandAll(const QModelIndex &idx)
 {
-    quint32 count=model()->rowCount();
+    quint32 count=model()->rowCount(idx);
     for (quint32 i=0; i<count; ++i) {
-        expand(model()->index(i, 0));
+        expand(model()->index(i, 0, idx));
     }
 }
 
@@ -153,6 +216,42 @@ void TreeView::expand(const QModelIndex &idx)
     }
 }
 
+bool TreeView::checkBoxClicked(const QModelIndex &idx) const
+{
+    QRect rect = visualRect(idx);
+    rect.moveTo(viewport()->mapToGlobal(QPoint(rect.x(), rect.y())));
+    int itemIndentation = rect.x() - visualRect(rootIndex()).x();
+    rect = QRect(header()->sectionViewportPosition(0) + itemIndentation, rect.y(), style()->pixelMetric(QStyle::PM_IndicatorWidth), rect.height());
+    return rect.contains(QCursor::pos());
+}
+
+void TreeView::setUseSimpleDelegate()
+{
+    setItemDelegate(new BasicItemDelegate(this));
+}
+
+void TreeView::setBackgroundImage(const QIcon &icon)
+{
+    QPalette pal=parentWidget()->palette();
+    if (!icon.isNull()) {
+        pal.setColor(QPalette::Base, Qt::transparent);
+    }
+    setPalette(pal);
+    viewport()->setPalette(pal);
+    bgnd=createBgndPixmap(icon);
+}
+
+void TreeView::paintEvent(QPaintEvent *e)
+{
+    if (!bgnd.isNull()) {
+        QPainter p(viewport());
+        QSize sz=size();
+        p.fillRect(0, 0, sz.width(), sz.height(), QApplication::palette().color(QPalette::Base));
+        p.drawPixmap((sz.width()-bgnd.width())/2, (sz.height()-bgnd.height())/2, bgnd);
+    }
+    QTreeView::paintEvent(e);
+}
+
 void TreeView::setModel(QAbstractItemModel *m)
 {
     QAbstractItemModel *old=model();
@@ -174,3 +273,17 @@ void TreeView::correctSelection()
     setCurrentIndex(currentIndex());
     selectionModel()->select(s, QItemSelectionModel::SelectCurrent);
 }
+
+void TreeView::itemWasActivated(const QModelIndex &index)
+{
+    if (!forceSingleClick) {
+        setExpanded(index, !isExpanded(index));
+    }
+}
+
+void TreeView::itemWasClicked(const QModelIndex &index)
+{
+    if (forceSingleClick) {
+        setExpanded(index, !isExpanded(index));
+    }
+}
