@@ -25,33 +25,52 @@
 #define TREEVIEW_H
 
 #include <QTreeView>
+#include <QPixmap>
+#include <QImage>
+
+class QIcon;
 
 class TreeView : public QTreeView
 {
     Q_OBJECT
 
 public:
+    static QImage setOpacity(const QImage &orig, double opacity=0.15);
+    static QPixmap createBgndPixmap(const QIcon &icon);
+    static void setForceSingleClick(bool v);
+    static bool getForceSingleClick();
+
     TreeView(QWidget *parent=0, bool menuAlwaysAllowed=false);
     virtual ~TreeView();
 
     void setPageDefaults();
+    void setExpandOnClick();
     void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
     bool haveSelectedItems() const;
     bool haveUnSelectedItems() const;
     void startDrag(Qt::DropActions supportedActions);
     void mouseReleaseEvent(QMouseEvent *event);
     QModelIndexList selectedIndexes() const;
-    void expandAll();
-    void expand(const QModelIndex &idx);
+    void expandAll(const QModelIndex &idx=QModelIndex());
+    virtual void expand(const QModelIndex &idx);
     virtual void setModel(QAbstractItemModel *m);
+    bool checkBoxClicked(const QModelIndex &idx) const;
+    void setUseSimpleDelegate();
+    void setBackgroundImage(const QIcon &icon);
+    void paintEvent(QPaintEvent *e);
 
 private Q_SLOTS:
     void correctSelection();
+    void itemWasActivated(const QModelIndex &index);
+    void itemWasClicked(const QModelIndex &index);
 
 Q_SIGNALS:
-    bool itemsSelected(bool);
+    void itemsSelected(bool);
+    void itemActivated(const QModelIndex &index); // Only emitted if view is set to single-click
+
 private:
     bool alwaysAllowMenu;
+    QPixmap bgnd;
 };
 
 #endif
